from recipe_scrapers.biancazapatka import BiancaZapatka
from tests import ScraperTest


class TestBiancaZapatkaScraper(ScraperTest):

    scraper_class = BiancaZapatka

    def test_host(self):
        self.assertEqual("biancazapatka.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Bianca Zapatka", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Rice Noodle Salad with Tempeh and Peanut Sauce",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual(
            "Lunch &amp; Dinner,Main Course", self.harvester_class.category()
        )

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://biancazapatka.com/wp-content/uploads/2022/07/buddha-bowl-sauce.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "7 oz Tempeh (oder Tofu)",
                "2 garlic cloves (pressed)",
                "3 piece of ginger (grated)",
                "3 tbsp tamari (or soy sauce)",
                "1 tbsp sesame oil (or peanut oil + more for frying)",
                "1 tbsp rice vinegar (or other vinegar or lemon/lime juice)",
                "1 tbsp maple syrup (or other syrup)",
                "1 tsp sriracha (or chili paste optional to taste)",
                "7 oz rice noodles (or other noodles or rice or quinoa)",
                "6 radishes (finely sliced)",
                "4 mini cucumbers (finely sliced)",
                "1 red bell pepper (finely sliced)",
                "2 large carrots (finely sliced)",
                "2 cups fresh baby spinach",
                "3-4 spring onions (cut into fine rings)",
                "1 recipe peanut sauce (or tahini sauce, if nut-free)",
                "2 tbsp sesame seeds",
                "4 tbsp roasted peanuts",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Marinated Tempeh\nFor the marinade, pour tamari, sesame oil, rice vinegar, maple syrup, and Sriracha into a sealable container. Press in the garlic, grate in the ginger, and stir everything together.\nCut the tempeh into cubes and add them to the marinade. Seal the container and shake well until the tempeh is completely marinated. Then let it marinate for at least 20 minutes (or overnight in the refrigerator).\nNoodles & Veggies\nMeanwhile, prepare and chop the vegetables. Cover the rice noodles with boiling water and set aside, covered, for 10 minutes (or according to package directions). Then drain and rinse with fresh water.\nCook and assemble\nOnce the tempeh is marinated, heat about 1-2 tbsp of oil in a frying pan (or wok). Remove the tempeh from the marinade (you can use the remaining marinade to make the peanut sauce) and cook until golden brown from all sides.\nAssemble the rice noodles and vegetables in bowls, as shown in the recipe video. Add the tempeh on top and drizzle with peanut sauce. Lastly, garnish with scallions, sesame seeds, and peanuts, and serve with fresh limes on the side.\nEnjoy!",
            self.harvester_class.instructions(),
        )

    def test_cuisine(self):
        self.assertEqual("Asian", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "This colorful Tempeh Rice Noodle Salad Bowl recipe is quick and easy to make, super healthy, and incredibly delicious! A vegan banquet of perfectly marinated tempeh with gluten-free rice noodles, crunchy fresh veggies, and the best peanut sauce ever! It’s the perfect bowl of happiness for anyone who’s looking to combine healthy eating with indulgence!",
            self.harvester_class.description(),
        )



from recipe_scrapers.lecker import Lecker
from tests import ScraperTest


class TestLeckerScraper2(ScraperTest):

    scraper_class = Lecker
    test_file_name = "lecker_2"

    def test_host(self):
        self.assertEqual("lecker.de", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            "Naan-Brot backen - so geht das Rezept",
            self.harvester_class.title(),
        )

    def test_image(self):
        self.assertEqual(
            "https://www.lecker.de/assets/field/image/naan-brot-b_0.jpg",
            self.harvester_class.image(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Was ist eigentlich Naan-Brot?Als Naan-Brot bezeichnet man dünne Teigfladen aus mit Joghurt gesäuertem Hefeteig, die traditionell über dem offenen Feuer gebacken werden. In unserem Rezept übernimmt diesen Part eine heiße Pfanne. Einfaches Naan-Brot wird ausschließlich mit Salz gewürzt und typischerweise als Beilage zu heißen Gerichten wie Currys und Fleischgerichten gereicht werden. Naan-Brot Rezept - Zutaten für 6 Stück:75 ml Milch1 TL Zucker1 TL Trockenhefe1/2 TL Salz250 g + etwas Mehl1 EL Öl75 g Vollmilchjoghurt1 Ei (Gr. M) Naan-Brot Rezept - Schritt 1:Erst kommt der Löffel - dann die Hände zum EinsatzDen Backofen auf ca. 50 °C vorheizen. 75 ml Milch lauwarm erwärmen und mit 1 TL Zucker sowie 1 TL Trockenhefe verrühren. Anschließend 25 Minuten ruhen lassen. 250 g Mehl und 1/2 TL Salz mischen. Hefemilch, 1 EL Öl, 75 g Joghurt und 1 Ei zugeben. Die Teigzutaten mit einem Kochlöffel verrühren. Dann mit den Händen zu einem geschmeidigen, glatten Teig verkneten. Naan-Brot Rezept - Schritt 2:Den Teig für das Naan-Brot mit einem Küchentuch abdecken und im warmen Ofen ca. 1 Stunde gehen lassen, bis sich das Volumen verdoppelt hat. Den Teig nach dem Gehen auf einer bemehlten Arbeitsfläche noch einmal kräftig durchkneten. Dadurch wird er elastischer und besser formbar. Anschließend zu 6 Kugeln formen und diese nochmals 15 Minuten ruhen lassen.Naan-Brot Rezept- Schritt 3:Das Brot lässt sich am besten mit dem Pfannenwender drehenDie Kugeln mit der Teigrolle jeweils zu ca. 1/2 cm dünnen ovalen Fladen à ca. 18 x 10 cm ausrollen. Eine Pfanne* ohne Fett erhitzen. Die Brotfladen darin bei starker Hitze nacheinander von jeder Seite 1-2 Minuten golbraun braten. Wenn das Brot blasen wirft, wenden und von der anderen Seite braten.Tipps: Ein Pfanne aus Gusseisen ist aufgrund ihrer natürlichen Beschichtung am besten zum Braten von Naan-Brot geeignet. Mit frischem Koriander, Knoblauch oder Kümmel im Grundteig sorgst du je nach Gusto im Handumdrehen für geschmackliche Vielfalt. Schnelle Teig-Variante mit BackpulverFür die Zubereitung des Hefeteigs muss man etwas Zeit einplanen. Wenn es mal schnell gehen soll, kann man auch eine Naan-Brot-Variante mit Backpulver backen. Für das Rezept 250 g Mehl, 1 TL Zucker, 1 TL Backpulver sowie einen 1/2 TL Salz in einer Schüssel mischen. 75 g Vollmilchjoghurt zugeben und alles zunächst mit einem Kochlöffel verrühren und anschließen mit den Händen zu einem geschmeidigen Teig verkneten.Aus dem Teig 6 gleich große Kugeln formen und diese auf bemehlter Arbeitsfläche mit der Teigrolle zu dünnen Fladen à ca. 18 x 10 cm ausrollen. Eine Pfanne erhitzen und die Brotfladen darin bei starker Hitze von jeder Seite 1-2 Minuten braten. Sobald das Brot blasen wirft, wenden und weiterbraten.Tipp: Für echte Brotliebhaber empfehlen wir einen Brotbackautomaten Und dazu schmeckt Naan-Brot besonders gut: Indisches Essen weiterlesen Curry-Rezepte weiterlesen",
            self.harvester_class.instructions(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [],
            self.harvester_class.ingredients(),
        )



from recipe_scrapers.cookingcircle import CookingCircle
from tests import ScraperTest


class TestCookingCircleScraper(ScraperTest):

    scraper_class = CookingCircle

    def test_host(self):
        self.assertEqual("cookingcircle.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Charlotte Roberts", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Vegan Lemon Cupcakes", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("30 mins or less", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://sharkninja-cookingcircle.s3.eu-west-1.amazonaws.com/wp-content/uploads/2022/02/23144350/Vegan-Lemon-Cupcakes-2220x1000.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "125ml non dairy milk",
                "1 tsp apple cider vinegar",
                "60ml olive oil",
                "100g caster sugar",
                "96g gluten free self raising flour",
                "1 tsp baking powder",
                "50g oat flour",
                "1 zest from lemon",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Step 1 Preheat your oven to 180c and line a cupcake tray with cases.",
                    "Step 2 Combine the non-dairy milk and cider vinegar, then set aside and allow to thicken.",
                    "Step 3 In your Ninja food processor, add in all of your dry ingredients and pulse to combine.",
                    "Step 4 Pour in the oil, milk mixture and lemon zest and blend for a few seconds to create a smooth batter.",
                    "Step 5 Transfer this into your cupcake cases (makes around 6 cupcakes) and bake in the oven for 25 minutes or until cooked through.",
                    "Step 6 Whilst allowing your cupcakes to cool, use an electric mixer to cream the butter until light and fluffy.",
                    "Step 7 Gradually add in the sugar and powders whilst mixing until fully combined.",
                    "Step 8 Once cooled, pipe the butter cream into each of your cakes and decorate.",
                ]
            ),
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class FoodRepublic(AbstractScraper):
    @classmethod
    def host(cls):
        return "foodrepublic.com"

    def title(self):
        return self.soup.find("h3", {"class": "recipe-title"}).get_text()

    def total_time(self):
        return sum(
            [
                get_minutes(self.soup.find("li", {"class": "prep-time"})),
                get_minutes(self.soup.find("li", {"class": "cook-time"})),
            ]
        )

    def yields(self):
        return get_yields(self.soup.find("span", {"itemprop": "recipeYield"}))

    def ingredients(self):
        ingredients = self.soup.findAll("li", {"itemprop": "recipeIngredient"})

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = self.soup.find("div", {"class": "directions"}).findAll("li")

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )



from recipe_scrapers.castironketo import CastIronKeto
from tests import ScraperTest


class TestCastIronKetoScraper(ScraperTest):

    scraper_class = CastIronKeto

    def test_host(self):
        self.assertEqual("castironketo.net", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.castironketo.net/blog/keto-jalapeno-popper-casserole-with-chicken/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Keto Jalapeño Popper Casserole with Chicken"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Cast Iron Keto")

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.castironketo.net/wp-content/uploads/2021/01/Keto-Jalapeño-Popper-Casserole.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 ½ lbs cooked shredded chicken breast",
                "4 green onions (chopped)",
                "1 teaspoon garlic salt",
                "½ teaspoon onion powder",
                "½ teaspoon paprika",
                "8 oz cream cheese (softened)",
                "½ cup heavy cream",
                "¼ cup chicken broth",
                "5 jalapeno peppers (halved (ribs removed if you like less spice))",
                "1 cup shredded sharp cheddar cheese",
                "6 slices bacon (cooked and crumbled)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 425° F.\nPlace the chicken in the bottom of a 10” or larger cast iron skillet. Top with green onions.\nCombine the garlic salt, onion powder, paprika, cream cheese, heavy cream, and chicken bone broth in a small bowl. Mix until combined.\nPour the cream cheese mixture over the chicken then top with the slices of jalapeno. Sprinkle the cheddar cheese over the top then top with the crumbled bacon.\nTransfer the skillet to the oven and bake for 15 minutes until the cheese is bubbly. Serve immediately.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.southerncastiron import SouthernCastIron
from tests import ScraperTest


class TestSouthernCastIronScraper(ScraperTest):

    scraper_class = SouthernCastIron

    def test_host(self):
        self.assertEqual("southerncastiron.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.southerncastiron.com/apple-cider-doughnuts/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Apple Cider Doughnuts")

    def test_yields(self):
        self.assertEqual("10 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.southerncastiron.com/wp-content/uploads/2018/10/doughnuts-featured-470x470.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 cups apple cider*",
                "2 tablespoons unsalted butter, softened",
                "1½ cups sugar, divided",
                "1 teaspoon vanilla extract",
                "1 large egg",
                "⅔ cup sour cream",
                "2¼ cups all-purpose flour",
                "1½ teaspoons baking powder",
                "1¼ teaspoons apple pie spice, divided",
                "½ teaspoon kosher salt",
                "½ cup grated Golden Delicious apple",
                "Vegetable oil, for frying",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In a small Dutch oven, bring apple cider to a boil over medium-high heat. Boil until cider is reduced to 1 cup, about 40 minutes. Remove from heat. Let cool to room temperature.\nIn a large bowl, beat butter, ½ cup sugar, and vanilla with a mixer at medium speed until combined. Add egg, beating until thick and pale, about 2 minutes. Beat in sour cream.\nIn a medium bowl, whisk together flour, baking powder, ¼ teaspoon apple pie spice, and salt. With mixer on low speed, gradually add flour mixture to butter mixture, beating until combined. Add apple, beating just until combined. Wrap dough in plastic wrap. (Dough will be slightly sticky.) Refrigerate for 30 minutes.\nOn a lightly floured surface, gently knead dough 3 or 4 times. Roll dough to ½ inch thick. Using a 3-inch cutter, cut dough, rerolling scraps as necessary.\nFill a cast-iron Dutch oven halfway full with oil, and heat over medium heat until a deep-fry thermometer registers 350°. Add doughnuts to hot oil in batches; fry until golden brown, about 1½ minutes per side. Fry doughnut holes for about 1 minute. Remove using a slotted spoon, and let drain on paper towels. Let cool slightly.\nIn a large bowl, combine remaining 1 cup sugar and remaining 1 teaspoon apple pie spice. Dip doughnut holes in sugar mixture, shaking off excess. Drizzle with apple cider syrup.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.zenbelly import ZenBelly
from tests import ScraperTest


class TestZenBellyScraper(ScraperTest):

    scraper_class = ZenBelly

    def test_host(self):
        self.assertEqual("zenbelly.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.zenbelly.com/gingerbread/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Paleo Gingerbread")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Simone Miller")

    def test_yields(self):
        self.assertEqual("20 servings", self.harvester_class.yields())

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_image(self):
        self.assertEqual(
            "https://www.zenbelly.com/wp-content/uploads/2019/01/gingerbread-3-225x225.jpeg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "butter (ghee, or shortening for greasing the pan)",
                "3 cups almond flour",
                "1 1/2 cups tapioca starch (plus more for flouring pan)",
                "1/2 cup coconut flour",
                "1 1/2 teaspoons baking soda",
                "2 teaspoons ground ginger",
                "1 teaspoons ground cinnamon",
                "1/4 teaspoon ground allspice",
                "1/4 teaspoon ground cardamom",
                "1/2 teaspoon finely ground sea salt",
                "1 cup coconut sugar",
                "1 cup molasses (use true molasses if you can find it)",
                "2 teaspoons fresh ginger",
                "1 cup boiling water",
                "4 eggs",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Preheat the oven to 350ºF. Grease a 9×13-inch cake pan.",
                    "In a large bowl, whisk together the almond flour, tapioca starch, coconut flour, baking soda, ground ginger, cinnamon, allspice, cardamom, and salt.",
                    "In a medium heat proof bowl, whisk together the coconut sugar, molasses, fresh ginger, and boiling water. Once it’s lukewarm (the molasses and coconut sugar should take the temperature down enough), whisk in the eggs.",
                    "Pour the wet ingredients into the dry ingredients and whisk until there are no lumps.",
                    "Pour into the prepared pan and bake for 28-35 minutes*",
                ]
            ),
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False

from recipe_scrapers.simplycookit import SimplyCookit
from tests import ScraperTest


class TestSimplyCookitScraper(ScraperTest):
    scraper_class = SimplyCookit

    def test_host(self):
        self.assertEqual("simply-cookit.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("simply-cookit.com", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Gnocchi mit Zuckerschoten und getrockneten Tomaten in Parmesansauce",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual("Vegetarisch", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(21, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.simply-cookit.com/sites/default/files/styles/square/public/assets/image/2021/03/gnocchi-mit-zuckerschoten-und-getrockneten-tomaten-in-parmesansosse_portrait.jpg?h=526df0cf&itok=wxxX6IM7",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "90 g Parmesan",
                "1 Zwiebel",
                "1 Knoblauchzehe",
                "0,5 TL Salz",
                "30 g Butter",
                "3 TL Weizenmehl, Type 405",
                "150 ml Gemüsefond",
                "250 ml Milch",
                "0,25 TL schwarzer Pfeffer, aus der Mühle",
                "800 g Gnocchi, aus dem Kühlregal",
                "100 g Tomaten, getrocknet, in Öl",
                "200 g Zuckerschoten",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Das Universalmesser einsetzen. Den Parmesan in ca. 3 cm großen Stücken in den Topf einwiegen. Den Deckel schließen, den Messbecher einsetzen und den Parmesan (Universalmesser | Stufe 18 | 20 Sek.) fein zerkleinern. Das Universalmesser entnehmen und das Lebensmittel mit dem Spatel abstreifen. Den Parmesan umfüllen und beiseitestellen.\nDas Universalmesser einsetzen. Die Zwiebel schälen, halbieren und in den Topf geben. Die Knoblauchzehe schälen und zugeben. Den Deckel schließen und die Zutaten (Universalmesser | Stufe 14 | 10 Sek.) zerkleinern. Das Universalmesser entnehmen und das Lebensmittel mit dem Spatel abstreifen. Die Zutaten mit dem Spatel nach unten schieben.\nIn einem Kochtopf reichlich Salzwasser zum Kochen bringen. In der Zwischenzeit den Zwillings-Rührbesen einsetzen. Die Butter in den Topf einwiegen.\nDen Deckel schließen, den Messbecher entnehmen und die Zwiebeln (Zwillings-Rührbesen | Stufe 3 | 120 °C | 4 Min.) andünsten. Das Mehl zugeben, den Deckel schließen und die Zutaten (Zwillings-Rührbesen | Stufe 3 | 120 °C | 3 Min.) anschwitzen. Den Gemüsefond und die Milch einwiegen sowie Salz und Pfeffer zugeben. Den Deckel schließen, den Messbecher einsetzen und die Zutaten (Zwillings-Rührbesen | Stufe 4 | 95 °C | 6 Min.) köcheln lassen.\nDie Gnocchi im Kochtopf nach Packungsangabe garen und anschließend abgießen. In der Zwischenzeit die getrockneten Tomaten in ca. 3 mm feine Stücke schneiden und in den Topf einwiegen. 60 g Parmesan einwiegen, den Deckel schließen und die Zutaten (Zwillings-Rührbesen | Stufe 4 | 100 °C | 1 Min.) vermengen. Den Zwillings-Rührbesen entnehmen. Die Zuckerschoten in ca. 2 cm große Stücke schneiden, einwiegen und mit dem Spatel unterheben.\nDie Sauce über die Gnocchi geben und mit dem restlichem Parmesan bestreut servieren.",
            self.harvester_class.instructions(),
        )

    def test_cuisine(self):
        self.assertEqual("Italien", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Ein besonders schnell zubereitetes Gericht - mit feiner Tomaten-Parmesansauce!",
            self.harvester_class.description(),
        )



from recipe_scrapers.ica import Ica
from tests import ScraperTest


class TestIcaScraper(ScraperTest):

    scraper_class = Ica

    def test_host(self):
        self.assertEqual("ica.se", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("ICA Köket", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Lysande gul fiskgryta", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("Huvudrätt,Middag", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://assets.icanet.se/t_ICAseAbsoluteUrl/imagevaultfiles/id_63144/cf_259/lysande_gul_fiskgryta.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "400 g torskfilé, sej eller hoki",
                "300 g laxfilé",
                "1/2 purjolök",
                "1/2 gul lök",
                "1 msk smör",
                "1 vitlöksklyfta",
                "1 1/2 tsk tomatpuré",
                "1 tsk timjan",
                "1 tsk basilika",
                "2 1/2 dl torrt vitt vin",
                "1 1/2 fiskbuljongtärning",
                "2 dl vispgrädde",
                "1 dl crème fraiche",
                "2 dl vatten",
                "1 pkt saffran (à 0,5 g)",
                "1 1/2 tsk salt",
                "300 g räkor med skal",
                "1 burk musslor (à 150 g)",
                "ev. bröd och vitlöksmajonnäs",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Tina fisken om den är fryst. Skölj och strimla purjon. Skala och hacka löken.\n"
            "Smält smöret i en stor gryta eller tjockbottnad kastrull. Fräs löken och purjol"
            "öken ett par minuter tills de blivit glansiga och genomskinliga. Pressa i vitlö"
            "k. Rör ner tomatpuré, timjan och basilika. Låt detta fräsa med en kort stund.\n"
            "Tillsätt vin och buljongtärning. Koka ett par minuter. Rör ner grädde, crème fr"
            "aiche, vatten och saffran. Sjud i 15 minuter. Smaka av med salt.\nSkär fisken i"
            " munsbitar, lägg i grytan och sjud i ytterligare 7 minuter.\nSkala räkorna. Til"
            "lsätt dem och de avrunna musslorna.\xa0\nServeringsförslag:\xa0Hetta upp och se"
            "rvera genast, gärna med bröd och vitlöksmajonnäs.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.5, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Denna fiskgryta får sin lysande solgula färg och ljuvliga smak av saffran, vitl"
            "ök och tomatpuré. Grytan blir matig och mättande med lax, torsk, räkor och muss"
            "lor och passar perfekt att bjuda på till en lite festligare middag med eller ut"
            "an gäster.",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper


class FranzoesischKochen(AbstractScraper):
    @classmethod
    def host(cls):
        return "franzoesischkochen.de"

    def author(self):
        # TODO: check to see whether the pages begin using 'name' (lowercase initial)
        # if they do, then we can use self.schema.author() instead here
        return self.schema.data.get("author").get("Name")

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        # TODO: can recipe yields / servings be retrieved from these pages?
        return None

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()



from recipe_scrapers.tastesoflizzyt import TastesOfLizzyT
from tests import ScraperTest


class TestTastesOfLizzyTScraper(ScraperTest):

    scraper_class = TastesOfLizzyT

    def test_host(self):
        self.assertEqual("tastesoflizzyt.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.tastesoflizzyt.com/soft-baked-gingerbread-cookies/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Soft Gingerbread Cookies")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Lizzy T")

    def test_total_time(self):
        self.assertEqual(27, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("60 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 cup shortening",
                "1 cup brown sugar (packed)",
                "1 cup molasses",
                "1 cup buttermilk",
                "5 1/2 cups flour",
                "4 teaspoons baking soda",
                "1 teaspoon ginger",
                "3/4 teaspoon cinnamon",
                "1/4 teaspoon nutmeg",
                "1/4 teaspoon cloves",
                "1 teaspoons salt",
                "1/2 cup extra sugar for rolling dough",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "In a large bowl, cream together the shortening, brown sugar, molasses and buttermilk.\nIn a separate bowl, sift together the flour, baking soda, ginger, cinnamon, nutmeg, cloves and salt.\nAdd the dry ingredients to the creamed sugar mixture and mix well.\nRoll the dough into balls and then roll the balls in sugar.\nPlace the cookie dough balls on an ungreased cookie sheet and bake at 350 degrees for 11-12 minutes.\nAllow the cookies to cool on a wire rack and then store in an airtight container.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper


class OwenHan(AbstractScraper):
    @classmethod
    def host(cls):
        return "owen-han.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.soup.find("h1", {"class": "entry-title"}).text

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return [x for x in map(lambda x: x.text, self.soup.select("ul > li"))]

    def instructions(self):
        return [x for x in map(lambda x: x.text, self.soup.select("ol > li"))]



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_host_name


class SchemaScraperFactory:
    class SchemaScraper(AbstractScraper):
        def host(self) -> str:  # type: ignore [override]
            return get_host_name(self.url) if self.url is not None else ""

        def title(self):
            return self.schema.title()

        def category(self):
            return self.schema.category()

        def total_time(self):
            return self.schema.total_time()

        def cook_time(self):
            return self.schema.cook_time()

        def prep_time(self):
            return self.schema.prep_time()

        def yields(self):
            return self.schema.yields()

        def image(self):
            return self.schema.image()

        def ingredients(self):
            return self.schema.ingredients()

        def instructions(self):
            return self.schema.instructions()

        def ratings(self):
            return self.schema.ratings()

        def author(self):
            return self.schema.author()

        def cuisine(self):
            return self.schema.cuisine()

        def description(self):
            return self.schema.description()

    @classmethod
    def generate(cls, url, **options):
        return cls.SchemaScraper(url, **options)



from recipe_scrapers.cdkitchen import CdKitchen
from tests import ScraperTest


class TestCdKitchen(ScraperTest):

    scraper_class = CdKitchen

    def test_host(self):
        self.assertEqual("cdkitchen.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.cdkitchen.com/recipes/recs/473/Veal-Steak-Vesuvio92364.shtml",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Veal Steak Vesuvio", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "FAKelco")

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 veal shoulder arm or blade steaks, cut 1 inch thick",
                "2 baking potatoes, cut lengthwise into 8 wedges",
                "1 lemon, cut lengthwise into 8 wedges",
                "1/2 cup frozen peas, cooked",
                "Seasoning",
                "2 tablespoons olive oil",
                "3 cloves garlic, minced",
                "1 teaspoon dried oregano",
                "1/2 teaspoon black pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Combine seasoning ingredients. Brush 1/2 of seasoning on veal steaks. Toss remaining seasoning with potatoes.\nPlace steaks and potatoes on rack in broiler pan so surface of veal is 3 to 4 inches from heat. Squeeze juice from lemon wedges over steaks and potatoes; place wedges on rack. Broil 26 to 28 minutes for medium doneness, turning steaks, potatoes and lemon once. Remove steaks.\nContinue broiling potatoes and lemon 3 to 5 minutes or until lightly browned. Carve veal; season with salt. Serve with potatoes, lemon and peas.",
            self.harvester_class.instructions(),
        )

    def test_image(self):
        self.assertEqual(
            "https://cdn.cdkitchen.com/recipes/images/sharing/25/veal-steak-vesuvio-63237-small.png",
            self.harvester_class.image(),
        )



from recipe_scrapers.mobkitchen import MobKitchen
from tests import ScraperTest


class TestMobKitchenScraper(ScraperTest):

    scraper_class = MobKitchen

    def test_host(self):
        self.assertEqual("mobkitchen.co.uk", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Mob", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Chilli Cheese Paratha", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("Munchies, Comfort, Speedy", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://mobkitchen-objects.imgix.net/recipes/9K8A6392-2.jpg?auto=format&crop=focalpoint&domain=mobkitchen-objects.imgix.net&fit=crop&fp-x=0.5&fp-y=0.5&h=827&ixlib=php-3.3.1&q=82&w=1300&s=412bb43eb2623e3afb2bebc5cc73b572",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "200g Plain Flour",
                "140ml Hot Water",
                "7 Garlic Cloves",
                "3 Tsp Chilli Powder",
                "Handful Of Coriander",
                "3 Spring Onions",
                "250g Mozzarella",
                "250g Cheddar Cheese",
                "Salt",
                "Vegetable Oil",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Combine the plain flour, a pinch of salt and hot water (from the kettle) in a bowl and mix with a wooden spoon. Once it is cool enough to handle, knead the dough for 5 minutes until smooth. Pour in a tablespoon of oil and knead through. Cover and rest for 10 minutes
Meanwhile, prepare the garlic chilli oil. Grate the garlic into a small microwaveable bowl with the chilli powder, ¼ teaspoon salt and 2 tablespoons of oil. Microwave for 1 minute to soften and cook the garlic.
Roughly chop your coriander and finely slice your spring onions.
Divide the dough into quarters, and then the quarters into halves to form eight even balls.
Roll two balls into large circles about 2mm thick so they are about the same size. Spread ½ a teaspoon of the chilli garlic paste on to one circle, sprinkle on a handful of shredded mozzarella and shredded cheddar. Sprinkle on your coriander and spring onions, then press a second dough circle on top – you can use some water along the edges to help it stick if you need it too
Heat up a frying pan and pour in a small glug of oil. Slap on the paratha and cook on a high heat on both sides for 2 minutes. Flip using a large spatula.
Cut into quarters and serve whilst still hot and oozing with cheese.""",
            self.harvester_class.instructions(),
        )

    def test_cuisine(self):
        self.assertEqual("Indian", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "This Indian snack is oozing with molten cheese and chilli garlicky goodness. They will be demolished in seconds, I can promise you that. Seema x",
            self.harvester_class.description(),
        )



from recipe_scrapers.cuisineaz import CuisineAZ
from tests import ScraperTest


class CuisineAZScraper(ScraperTest):

    scraper_class = CuisineAZ

    def test_host(self):
        self.assertEqual("cuisineaz.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.cuisineaz.com/recettes/filet-de-saumon-au-four-63049.aspx",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Filet de saumon au four")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Cuisine AZ")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://img.cuisineaz.com/660x660/2014/02/24/i78436-filet-de-saumon-au-four.jpeg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "600 g Filet de saumon",
                "30 ml Huile d'olive",
                "4 g Origan, séché",
                "1 pincée(s) sel",
                "1 pincée(s) Poivre",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Etape 1\nPréchauffez le four th.7 (210°C). Coupez le filet en pavés de saumon de même taille, correspondant au nombre de parts désirées, et déposez-les sur une plaque huilée ou dans un plat à four. Arrosez-les d'huile d'olive. Salez et poivrez à votre convenance et parsemez d'origan.\nEtape 2\nDisposez le plat ou la plaque au centre du four. Comptez environ 10 min pour un filet de 2 à 2,5 cm d'épaisseur. Le temps nécessaire à la bonne cuisson du saumon dépend non seulement de l'épaisseur des pavés mais aussi de la température réelle de votre four, c'est pourquoi il est important de vérifier régulièrement la cuisson du saumon à l'aide d'une fourchette.\nEtape 3\nLorsque vos pavés de saumon sont cuits, servez-les immédiatement, accompagnés d'une bonne salade bien assaisonnée, et éventuellement d'une belle timbale de riz basmati.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class Vegolosi(AbstractScraper):
    @classmethod
    def host(cls):
        return "vegolosi.it"

    def title(self):
        return self.soup.find("h1").get_text().strip()

    def author(self):
        return self.schema.author()

    def preparation_time(self):
        possible_time_info_elements = self.soup.findAll(
            "span", {"class": "tasty-recipes-prep-time"}
        )

        return sum([get_minutes(element) for element in possible_time_info_elements])

    def cooking_time(self):
        possible_time_info_elements = self.soup.findAll(
            "span", {"class": "tasty-recipes-cook-time"}
        )

        return sum([get_minutes(element) for element in possible_time_info_elements])

    def total_time(self):
        possible_time_info_elements = self.soup.findAll(
            "span", {"class": "tasty-recipes-total-time"}
        )

        return sum([get_minutes(element) for element in possible_time_info_elements])

    def yields(self):
        possible_yields_info_elements = self.soup.findAll(
            "span", {"class": "tasty-recipes-yield"}
        )
        for element in possible_yields_info_elements:
            if "persone" in element.get_text():
                return get_yields(element)

    def ingredients(self):
        ingredients = self.soup.select(".tasty-recipe-ingredients > ul > li")

        if not ingredients:
            ingredients = self.soup.findAll("li", {"class": "ingredient"})

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):

        instructions = self.soup.findAll("div", {"class": "tasty-recipe-instructions"})

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def ratings(self):
        return round(
            float(
                self.soup.find("div", {"class": "tasty-recipe-rating rating_panel"})
                .get("data-content-rate")
                .replace(",", ".")
            ),
            2,
        )



from recipe_scrapers.hostthetoast import Hostthetoast
from tests import ScraperTest


class TestHostthetoastScraper(ScraperTest):

    scraper_class = Hostthetoast

    def test_host(self):
        self.assertEqual("hostthetoast.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://hostthetoast.com/homemade-garlic-naan/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Homemade Garlic Naan")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Morgan")

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://hostthetoast.com/wp-content/uploads/2018/08/naan-202-320x320-1-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1/4 cup warm water",
                "1 tablespoon sugar",
                "1 package (2 1/4 teaspoons) active dry yeast",
                "3/4 cup warm milk",
                "3/4 cup plain yogurt",
                "4 cups all-purpose flour",
                "1 teaspoon Kosher salt",
                "1 stick melted butter, for brushing",
                "4 cloves minced garlic",
                "Fresh cilantro, to top",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In a glass measuring cup, combine the yeast, sugar, and water and let sit until very foamy, about 10 minutes. Meanwhile, whisk the flour and salt together in a large bowl and create a well in the center.\nWhisk in the warm milk and plain yogurt into the yeast mixture until well-combined. Pour into the well in the dry ingredients. Stir until a dough is formed, then turn out onto a lightly-floured surface and knead until smooth, about 3-4 minutes. Transfer the dough to a large, lightly oiled bowl and cover loosely with a damp kitchen towel. Let rise at room temperature until doubled in size, about 1 hour.\nTurn the dough out onto a floured surface. Knead briefly into a disc and cut the dough into 12 equal-sized pieces. Roll each piece into a ball.\nHeat a large, heavy bottomed skillet over medium heat. Roll each dough ball out until it is about 1/4 inch thick and approximately 6 inches wide. Brush the dough lightly with butter and place one at a time onto the hot skillet. Cook until large bubbles form on the surface, about 2 minutes. Flip the dough and cook the other side until golden, about 1-2 more minutes. Stack the cooked flat bread on a plate and cover with a towel to keep warm as you cook the remaining pieces.\nAdd the minced garlic to the remaining melted butter. Loosely cover and microwave for 15 seconds. Brush the warm naan with the garlic butter (scooping out some of the garlic to sit on top) and sprinkle generously with cilantro. Serve warm.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.reishunger import Reishunger
from tests import ScraperTest


class TestReishungerScraper(ScraperTest):

    scraper_class = Reishunger
    test_file_name = "reishunger_1"

    def test_host(self):
        self.assertEqual("reishunger.de", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("pommesherz", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Crispy Tofu Bowl", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn.reishunger.com/crispy-tofu-bowl.jpg?quality=85",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "200 gr Bio Basmati Reis",
                "50 ml Erdnuss Sauce",
                "200 gr Tofu",
                "etwas Paniermehl",
                "etwas Mehl",
                "1 Ei",
                "100 gr Edamame",
                "1 Möhre",
                "1/4 Rotkohl",
                "50 gr Zuckerschoten",
                "5 Maiskölbchen",
                "30 gr Sprossen",
                "eine Handvoll Cashew Kerne",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Reis nach Anleitung im Digitalen Reiskocher oder Kochtopf kochen.",
                    "Gemüse schneiden und ca. 5 min blanchieren.",
                    "Den Tofu in Stücke schneiden und in Mehl Ei und Paniermehl wälzen und kurz von allen Seiten anbraten.",
                    "Cashew Kerne in einer Pfanne kurz anrösten.",
                    "Die Sauce erwärmen und den Tofu auf einen Spieß ziehen (optimal).",
                    "Alles schön anrichten und mit den Sprossen toppen. Guten Reishunger! :-)",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.67, self.harvester_class.ratings())



from recipe_scrapers.bettycrocker import BettyCrocker
from tests import ScraperTest


class TestBettyCrocker(ScraperTest):

    scraper_class = BettyCrocker

    def test_host(self):
        self.assertEqual("bettycrocker.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.bettycrocker.com/recipes/eclair-bars/0cb1d3f0-f074-400b-aa5a-5e729102c4ec",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Éclair Bars")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Betty Crocker Kitchens")

    def test_total_time(self):
        self.assertEqual(290, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("24 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://images-gmi-pmc.edge-generalmills.com/e1306219-6178-4e15-afaf-ba6e527f1e6b.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        return self.assertEqual(
            [
                "1 can (8 oz) refrigerated Pillsbury™ Original Crescent Dough Sheet",
                "2 boxes (3.4 oz each) Jell-O™ vanilla-flavor instant pudding & pie filling mix",
                "3 cups cold half-and-half",
                "1 1/2 cups semisweet chocolate chips",
                "3/4 cup heavy whipping cream",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Heat oven to 375°F. Spray bottom only of 13x9-inch pan with cooking spray.",
                    "Unroll crescent dough; press in bottom of pan. Bake 12 to 14 minutes or until golden brown and baked through. Remove from oven to cooling rack; cool 20 minutes.",
                    "In medium bowl, beat dry pudding mixes and half-and-half with whisk about 2 minutes or until thick. Spread over cooled bar base.",
                    "In medium microwavable bowl, microwave chocolate chips and whipping cream uncovered on High 1 minute; stir. Microwave 30 seconds; stir until smooth. Carefully spread mixture on top of pudding layer. Refrigerate about 4 hours or until cooled completely.",
                    "When ready to serve, using a sharp knife and up-and-down sawing motion for cleaner cuts, cut into 6 rows by 4 rows. Store covered in refrigerator.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.5, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class JamieOliver(AbstractScraper):
    @classmethod
    def host(cls):
        return "jamieoliver.com"

    def title(self):
        return self.soup.find("h1").get_text()

    def total_time(self):
        return get_minutes(self.soup.find("div", {"class": "time"}))

    def yields(self):
        return get_yields(self.soup.find("div", {"class": "recipe-detail serves"}))

    def image(self):
        container = self.soup.find("div", {"class": "recipe-header-left"})
        if not container:
            return None

        image = container.find("img", {"src": True})
        return image["src"] if image else None

    def ingredients(self):
        ingredients = self.soup.find("ul", {"class", "ingred-list"}).findAll("li")
        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = self.soup.find("ol", {"class": "recipeSteps"}).findAll("li")
        return "\n".join([normalize_string(inst.get_text()) for inst in instructions])



from recipe_scrapers.yemek import Yemek
from tests import ScraperTest


class TestYemekScraper(ScraperTest):

    scraper_class = Yemek

    def test_host(self):
        self.assertEqual("yemek.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Yasemin Gürsürer", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Köri Soslu Tavuklu Patates Topları", self.harvester_class.title()
        )

    def test_total_time(self):
        self.assertEqual(85, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn.yemek.com/mncrop/300/200/uploads/2020/06/kori-soslu-tavuklu-patates-toplari-yemekcom.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 yemek kaşığı sıvı yağ",
                "1 kilogram tavuk",
                "1 adet soğan",
                "1 adet kapya biber",
                "300 gram mantar",
                "1,5 çay kaşığı tuz",
                "1 çay kaşığı karabiber",
                "1 çay kaşığı toz kırmızı biber",
                "1 çay bardağı su",
                "5 adet patates",
                "2 yemek kaşığı tereyağı",
                "2 çay kaşığı tuz",
                "2 yemek kaşığı tereyağı",
                "1 tatlı kaşığı sıvı yağ",
                "2 yemek kaşığı un",
                "2,5 su bardağı süt",
                "1 tatlı kaşığı köri",
                "1/2 çay kaşığı tuz",
                "100 gram rendelenmiş mozarella peyniri",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Patatesleri bir tencerede haşlayın. Haşlandıktan sonra suyunu süzüp tereyağı ve tuzla tatlandırıp püre haline getirin. Kişi sayısına göre eşit parçalara bölüp top haline getirin.\nTavukları pişireceğiniz tavayı ısıtın ve sıvı yağ ekleyip tavukları suyunu salıp çekene dek pişirin.\nSoğanları da tavukların üzerine ekleyip 2-3 dakika soteleyin. Ardından kapya biberi de ilave edip kavurun ve mantarları ekleyip suyunu salıp çekmesini bekleyin. Ara ara karıştırın.\nTuz, karabiber ve kırmızı biberi de ilave edip karıştırın ve son olarak suyunu ilave edip kısık ateşte 10 dakika kadar pişirin.\nSos için, tereyağını bir sos tenceresinde eritin ve sıvı yağ ekleyin. Üzerine un ve köriyi ilave edip 2-3 dakika kavurun. Sütü de yavaş yavaş ilave edip sürekli olarak karıştırın. Kaynamaya ve kıvamı koyulaşmaya başladıktan sonra altını kısıp 5 dakika daha pişirip ocaktan alın. Beklerken çok koyulaşırsa su veya sütle açın.\nFırını 200 dereceye ayarlayın.\nBorcama pişirdiğiniz sebzeli tavuğu yayın. Üzerine hazırladığınız patates toplarını, aralarında boşluk olacak şekilde dizin. Her bir patates topunun üzerine köri sosundan eşit miktarda dökün. Son olarak üzerlerine rendelenmiş mozarella ilave edip 200 derece fırında üzerleri kızarana dek 15 dakika pişirin.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.5, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class AllRecipes:
    @classmethod
    def host(cls):
        return "allrecipes.com"

    def __new__(cls, url, *args, **kwargs):
        if AllRecipesUser.host() in url:
            return AllRecipesUser(url, *args, **kwargs)
        return AllRecipesCurated(url, *args, **kwargs)


class AllRecipesCurated(AbstractScraper):
    @classmethod
    def host(cls):
        return "allrecipes.com"

    def author(self):
        # NB: In the schema.org 'Recipe' type, the 'author' property is a
        # single-value type, not an ItemList.
        # allrecipes.com seems to render the author property as a list
        # containing a single item under some circumstances.
        # In those cases, the SchemaOrg class will fail due to the unexpected
        # type, and this method is called as a fallback.
        # Rather than implement non-standard handling in SchemaOrg, this code
        # provides a (hopefully temporary!) allrecipes-specific workaround.
        author = self.schema.data.get("author")
        if author and isinstance(author, list) and len(author) == 1:
            author = author[0].get("name")
        return author

    def title(self):
        return self.schema.title()

    def description(self):
        return self.schema.description()

    def cook_time(self):
        return self.schema.cook_time()

    def prep_time(self):
        return self.schema.prep_time()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def category(self):
        return self.schema.category()


class AllRecipesUser(AbstractScraper):
    """Parse "unpublished" personal recipes on AllRecipes.com.

    Unpublised recipes are not structured, unlike curated recipes.
    Certain information is not always available, as it relies on what the
    users provided.
    """

    @classmethod
    def host(cls):
        return "allrecipes.com/cook"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # meta contains servings and yields, as well as preparation, cooking,
        # and total times
        # Possible keys are 'servings', 'yield', 'cook', 'prep', 'total'
        meta = zip(
            self.soup.findAll("div", {"class": "recipe-meta-item-header"}),
            self.soup.findAll("div", {"class": "recipe-meta-item-body"}),
        )
        self.meta = {
            k.text.lower().rstrip(":"): normalize_string(v.text) for k, v in meta
        }

    def title(self):
        title = self.soup.find("h1", {"class": "heading-content"}).text
        return title

    def total_time(self):
        if "total" in self.meta:
            total_time = get_minutes(self.meta["total"], return_zero_on_not_found=True)
        else:
            total_time = get_minutes(self.meta.get("cook", 0)) + get_minutes(
                self.meta.get("prep", 0)
            )
        return total_time

    def yields(self):
        yield_data = self.meta.get("yield")
        if yield_data is None:
            yield_data = self.meta.get("servings")
        return get_yields(yield_data)

    def image(self):
        image = self.soup.find("div", {"class": "lead-media", "data-src": True})
        image = image.get("data-src")
        return image

    def nutrients(self):
        raise NotImplementedError("Not available for personal recipes.")

    def ingredients(self):
        ingredients = self.soup.findAll("span", {"class": "ingredients-item-name"})
        ingredients = [normalize_string(i.text) for i in ingredients]
        return ingredients

    def instructions(self):
        instructions = self.soup.findAll("div", {"class": "paragraph"})
        instructions = "\n".join(normalize_string(i.text) for i in instructions)
        return instructions

    def ratings(self):
        lacks_rating = self.soup.find("a", {"class": "no-ratings"})
        if lacks_rating:
            return None

        ratings = self.soup.find("span", {"class": "review-star-text"})
        return float(ratings.text.lstrip("Ratings:").rstrip("stars"))

    def author(self):
        author = self.soup.find("span", {"class": "author-name"}).text
        return author

    def reviews(self):
        """Return a list of dictionaries containing reviews.

        Each dictionary contains the author, datePublished, reviewRating, reviewBody.
        These keys follow the JSON-LD format.
        """
        reviews = []
        for element in zip(
            self.soup.findAll("span", {"class": "reviewer__name"}),  # name
            self.soup.findAll("span", {"class": "feedback__reviewDate"}),  # date
            self.soup.findAll("span", {"class": "review-star-text"}),  # rating
            self.soup.findAll("div", {"class": "feedback__reviewBody"}),  # comment
        ):
            name, date, rating, comment = element
            name = normalize_string(name.text)
            date = date.text
            # The value is "Ratings: 4 stars"
            rating = float(rating.text.lstrip("Ratings:").rstrip("stars"))
            comment = normalize_string(comment.text)

            reviews.append(
                {
                    "author": name,
                    "datePublished": date,
                    "reviewRating": rating,
                    "reviewBody": comment,
                }
            )
        return reviews

    def cuisine(self):
        return None

    def category(self):
        return None



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, normalize_string


class JustBento(AbstractScraper):
    @classmethod
    def host(cls):
        return "justbento.com"

    def title(self):
        expected_prefix = "Recipe: "
        title = self.soup.find("meta", {"property": "og:title", "content": True})
        return title.get("content").replace(expected_prefix, "")

    def total_time(self):
        time = self.soup.find(
            "div", {"class": "field-name-taxonomy-vocabulary-2"}
        ).find("a", {"typeof": "skos:Concept"})
        return get_minutes(time)

    def yields(self):
        return "1"

    def ingredients(self):
        ingredients = (
            self.soup.find("div", {"class": "field-name-body"}).find("ul").findAll("li")
        )
        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        elements_after_title = (
            self.soup.find("div", {"class": "field-name-body"})
            .find("h3")
            .find_next_sibling("ul")
            .find_next_siblings()
        )

        instructions = []
        for element in elements_after_title:
            if instructions and element.name != "p":
                break
            if element.name == "p":
                instructions.append(element.get_text())
            instructions = [
                normalize_string(instruction) for instruction in instructions
            ]

        return "\n".join(instructions) if instructions else None

    def image(self):
        image = self.soup.find("div", {"class": "field-name-body"}).find(
            "img", {"class": "centerimg", "src": True}
        )
        return image["src"] if image else None



from recipe_scrapers.fitmencook import FitMenCook
from tests import ScraperTest

# test recipe's URL
# https://fitmencook.com/healthy-chili-recipe/


class TestFitMenCookScraper(ScraperTest):

    scraper_class = FitMenCook

    def test_host(self):
        self.assertEqual("fitmencook.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://fitmencook.com/healthy-chili-recipe/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Lean Chili with Plantains", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "FitMenCook")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("5 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 tablespoon avocado oil (or olive oil)",
                "1 tablespoon garlic, minced",
                "2/3 cup onion, diced",
                "1 bell pepper, diced",
                "1 1/2 lb 95% lean beef (I used lean bison)",
                "Seasonings 1 tablespoon chili powder 2 teaspoons smoked paprika 2 teaspoons "
                "cumin 1/2 teaspoon cinnamon 2 teaspoons (Mexican) oregano",
                "1 tablespoon chili powder",
                "2 teaspoons smoked paprika",
                "2 teaspoons cumin",
                "1/2 teaspoon cinnamon",
                "2 teaspoons (Mexican) oregano",
                "2 cups low sodium beef stock",
                "4 tablespoons tomato paste (NOT sauce)",
                "1 can (14.5oz /411g) no salt added diced tomatoes",
                "1 ripe plantain (~200g), cut into 1-inch pieces",
                "sea salt & pepper to taste",
                "Garnish cilantro jalapeño",
                "cilantro",
                "jalapeño",
                "1 cucharada de aceite de aguacate (o aceite de oliva)",
                "1 cucharada de ajo picado",
                "2/3 taza de cebolla, cortada en cubitos",
                "1 pimentón, cortado en cubitos",
                "1 1/2 lb 95% de carne magra (utilicé bisonte magro)",
                "Condimentos 1 cucharada de chile en polvo 2 cucharaditas de pimentón ahumado "
                "2 cucharaditas de comino 1/2 cucharadita de canela 2 cucharaditas de orégano "
                "(mexicano) 2 tazas de caldo de res bajo en sodio 4 cucharadas de pasta de "
                "tomate (NO salsa) 1 lata (13.5 oz) de tomates cortados en cubitos",
                "1 cucharada de chile en polvo",
                "2 cucharaditas de pimentón ahumado",
                "2 cucharaditas de comino",
                "1/2 cucharadita de canela",
                "2 cucharaditas de orégano (mexicano)",
                "2 tazas de caldo de res bajo en sodio",
                "4 cucharadas de pasta de tomate (NO salsa)",
                "1 lata (13.5 oz) de tomates cortados en cubitos",
                "1 plátano maduro grande, cortado en trozos de 1 pulgada",
                "sal marina y pimienta al gusto",
                "Adornos cilantro jalapeño",
                "cilantro",
                "jalapeño",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Set a pot on medium heat. Once hot, add the oil, garlic, onion and bell pepper. Cook for 2 – 3 minutes until the onion is browned along the edges.\nAdd the beef and chop it up finely as it cooks in the pot for 3 – 5 minutes. Sprinkle in the seasoning as the meat cooks.\nAdd the remaining ingredients EXCEPT plantain and stir everything together and bring to a light simmer. Cover and cook for 10 minutes.\nAdd the plantain after 10 minutes and stir everything together. ONLY if needed, add tablespoons of beef stock (or water) to the chili if it’s too thick, but also monitor the heat and make sure it’s not too high. Cover and cook for an additional 10 minutes.\nSeason to taste with sea salt & pepper (and lime) and garnish with cilantro and jalapeño.\nPon una olla a fuego medio. Una vez este caliente, agregue el aceite, el ajo, la cebolla y el pimentón. Cocine durante 2 a 3 minutos hasta que la cebolla se dore por los bordes.\nAgregue la carne y córtela finamente mientras se cocina en la olla durante 3 a 5 minutos. Espolvorea el condimento mientras la carne se cocina.\nAgregue los ingredientes restantes EXCEPTO el plátano y revuelva todo junto y cocine a fuego lento. Cubra y cocine por 10 minutos.\nAgregue el plátano después de 10 minutos y revuelva todo junto. SÓLO si es necesario, agregue cucharadas de caldo de carne (o agua) al chile si es demasiado espeso, pero también controle el calor y asegúrese de que no sea demasiado alto. Cubra y cocine por 10 minutos adicionales.\nSazone al gusto con sal marina y pimienta (y lima) y adorne con cilantro y jalapeño.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import normalize_string


class SweetPeasAndSaffron(AbstractScraper):
    """
    Web scraper for Sweet Peas & Saffron website
    """

    @classmethod
    def host(cls):
        return "sweetpeasandsaffron.com"

    def title(self):
        return self.soup.find("h2", {"class": "wprm-recipe-name"}).text

    def total_time(self):
        return self.soup.find("span", {"class": "wprm-recipe-total_time"}).text

    def yields(self):
        return self.soup.find("span", {"class": "wprm-recipe-servings"}).text

    def image(self):
        return self.soup.find(
            "div", {"class": "wprm-recipe-image wprm-block-image-normal"}
        ).find("img")["data-src"]

    def _instructions_list(self):
        instructions = [
            e.text
            for e in self.soup.find_all(
                "div", {"class": "wprm-recipe-instruction-text"}
            )
        ]
        return [normalize_string(instruction) for instruction in instructions]

    def instructions(self):
        data = self._instructions_list()
        return "\n".join(data) if data else None

    def ingredients(self):
        ingredient_amount = self.soup.find_all(
            "span", {"class": "wprm-recipe-ingredient-amount"}
        )
        ingredient_unit = self.soup.find_all(
            "span", {"class": "wprm-recipe-ingredient-unit"}
        )
        ingredient_name = self.soup.find_all(
            "span", {"class": "wprm-recipe-ingredient-name"}
        )
        ingredients = [
            " ".join([n.text, a.text, u.text])
            for a, u, n in zip(ingredient_amount, ingredient_unit, ingredient_name)
        ]
        return ingredients

    def ratings(self):
        return self.soup.find("span", {"class": "wprm-recipe-rating-average"}).text

    def author(self):
        return self.soup.find("span", {"class": "wprm-recipe-author"}).text



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class NHSHealthierFamilies(AbstractScraper):
    @classmethod
    def host(cls):
        return "nhs.uk"

    def author(self):
        return "NHS Better Health"

    def title(self):
        title = normalize_string(self.soup.find("h1").get_text())
        if title.endswith(" recipe"):
            title = title[:-7]
        return title

    def _get_recipe_metadata(self):
        container = self.soup.find("div", {"class": "bh-recipe__description"})
        descriptions = container.findAll("p")
        content = "".join([description.get_text() for description in descriptions])
        prep_time = re.search(r"Prep: (\d+) mins", content)
        cook_time = re.search(r"Cook: (\d+) mins", content)
        recipe_yields = re.search(r"Serves (\d+)", content)
        return {
            "prep_time": get_minutes(prep_time.group(0)) if prep_time else None,
            "cook_time": get_minutes(cook_time.group(0)) if cook_time else None,
            "yields": get_yields(recipe_yields.group(0)) if recipe_yields else None,
        }

    def total_time(self):
        metadata = self._get_recipe_metadata()
        return metadata["prep_time"] + metadata["cook_time"]

    def yields(self):
        metadata = self._get_recipe_metadata()
        return metadata["yields"]

    def image(self):
        return self.soup.find("img", {"class": "nhsuk-image__img"})["src"]

    def ingredients(self):
        ingredients = (
            self.soup.find("div", {"class": "bh-recipe-instructions"})
            .find("ul")
            .findAll("li")
        )
        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = (
            self.soup.find("div", {"class": "bh-recipe-instructions"})
            .find("ol")
            .findAll("li")
        )
        instructions = [
            instruction.find("p").get_text() for instruction in instructions
        ]
        return "\n".join(
            [normalize_string(instruction) for instruction in instructions]
        )

    def description(self):
        description_meta = self.soup.find("meta", {"name": "description"})
        return normalize_string(description_meta["content"])



from recipe_scrapers.jamieoliver import JamieOliver
from tests import ScraperTest


class TestJamieOliverScraper(ScraperTest):

    scraper_class = JamieOliver

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.jamieoliver.com/recipes/chocolate-recipes/bloomin-brilliant-brownies/",
            self.harvester_class.canonical_url(),
        )

    def test_host(self):
        self.assertEqual("jamieoliver.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Bloomin' brilliant brownies")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Jamie Oliver")

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("20 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://img.jamieoliver.com/jamieoliver/recipe-database/oldImages/large/88_1_1441269331.jpg?tr=w-800,h-1066",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "200 g quality dark chocolate (70%)",
                "250 g unsalted butter",
                "75 g dried sour cherries , optional",
                "50 g chopped nuts , optional",
                "80 g quality cocoa powder",
                "65 g plain flour",
                "1 teaspoon baking powder",
                "360 g caster sugar",
                "4 large free-range eggs",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Preheat the oven to 180°C/350°F/gas 4. Line a 24cm square baking tin with greaseproof paper.",
                    "Snap the chocolate into a large bowl, add the butter and place over a pan of simmering water, until melted, stirring regularly. Stir through the cherries and nuts (if using).",
                    "Sift the cocoa powder and flour into a separate bowl, add the baking powder and sugar, then mix together.",
                    "Add the dry ingredients to the chocolate, cherry and nut mixture and stir together well. Beat the eggs, then mix in until you have a silky consistency.",
                    "Pour the brownie mix into the baking tin, and place in the oven for around 25 minutes. You don’t want to overcook them so, unlike cakes, you don’t want a skewer to come out clean – the brownies should be slightly springy on the outside but still gooey in the middle.",
                    "Allow to cool in the tray, then carefully transfer to a large chopping board and cut into chunky squares. Delicious served with a dollop of crème fraîche or yoghurt mixed with some orange zest.",
                ]
            ),
            self.harvester_class.instructions(),
        )



from recipe_scrapers.lekkerensimpel import LekkerEnSimpel
from tests import ScraperTest


class TestLekkerEnSimpelScraper(ScraperTest):

    scraper_class = LekkerEnSimpel

    def test_host(self):
        self.assertEqual("lekkerensimpel.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual("Poké bowl met kip", self.harvester_class.title())

    def test_image(self):
        self.assertEqual(
            "https://www.lekkerensimpel.com/wp-content/uploads/2022/03/588A2370-1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "200 gr sushi rijst",
                "2 el sushi azijn (of 1,5 el rijstazijn, 1 tl suiker en een snuf zout)",
                "2 krokante kipschnitzels",
                "1 avocado",
                "100 gr peen julienne",
                "0.5 komkommer",
                "2 el gebakken uitjes",
                "Japanse mayonaise",
                "sojasaus",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertIn(
            "Begin met het maken van de sushirijst. Wij houden hiervoor dit sushirijst recept aan. Tip: heb je weinig tijd? Gebruik dan pandan rijst in plaats van sushi rijst. Dit is een stuk sneller klaar.\nSnijd de avocado en komkommer in plakjes/reepjes.\nGiet een scheutje olie in een koekenpan en bak de kipschnitzels volgens de instructies op het pak. Tip: maak ook eens de kipschnitzels zelf. Snijd de kipschnitzels daarna in plakjes.\nVerdeel de rijst over twee kommen. Verdeel de komkommer, kip, peen julienne, avocado en gebakken uitjes hier overheen. Als laatste verdeel je wat Japanse mayonaise en sojasaus over het geheel.",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "Een heerlijke poké bowl met kip, avocado, peen julienne en edamame bonen. Een makkelijk gerecht dat in 30 minuten op tafel staat én waarmee je heel goed kunt variëren.",
            self.harvester_class.description(),
        )

    def test_language(self):
        self.assertEqual("nl-NL", self.harvester_class.language())



# mypy: allow-untyped-defs

from recipe_scrapers.maangchi import Maangchi
from tests import ScraperTest


class TestMaangchiScraper(ScraperTest):

    scraper_class = Maangchi

    def test_host(self):
        self.assertEqual("maangchi.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual(None, self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Pepper fried chicken (Yuringi: 유린기)", self.harvester_class.title()
        )

    def test_category(self):
        self.assertEqual("Main", self.harvester_class.category())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://i.ytimg.com/vi/JhE4ToJOPcc/maxresdefault.jpg?meta=og:image",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 pound boneless & skinless chicken thighs",
                "½ teaspoon kosher salt",
                "¼ teaspoon ground black pepper",
                "2 inch of white part of dae-pa (or 2 to 3 regular green onions), thinly sliced lengthwise",
                "¼ of small onion (about 2 tablespoons), thinly sliced",
                "2 cups grape seeds oil (or vegetable oil, corn oil) for frying",
                "4 to 5 lettuce leaves",
                "3 to 4 spicy green and red chili peppers (or non-spicy peppers, green and red bell pepper), thinly sliced",
                "2 tablespoons grape seeds oil (or vegetable oil)",
                "1 teaspoon crushed chili pepper (or 2 small dried red chili peppers or Korean hot pepper flakes)",
                "3 tablespoons soy sauce",
                "3 tablespoons white vinegar (or apple cider vinegar)",
                "3 tablespoons Turbinado sugar (or white or brown sugar)",
                "2 tablespoons water",
                "4 garlic cloves, minced",
                "1 teaspoon toasted sesame oil",
                "¼ cup potato starch",
                "⅛ teaspoon kosher salt",
                "1 large egg",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Pat the chicken thighs dry with paper towels. Score a crosshatch patten on both sides of the chicken pieces.\nPound the thighs to an even thickness with a meat tenderizer or with the back of your kitchen knife.\nSprinkle both sides with salt and ground black pepper.\nSet aside or refrigerate until ready to use.\nPlace the shredded green onion and onion in a bowl and add cold water. Let it sit.\nHeat the vegetable oil in a small pan until it begins to smoke, about 30 seconds to 1 minute.\nRemove from the heat. Stir in the pepper flakes until they are slightly brown. Strain the oil into a medium sized stainless steel bowl.\nAdd the soy sauce, vinegar, sugar, water, garlic, and sesame oil.\nStir well until the sugar is completely dissolved.\nPrepare a shallow and wide bowl (I use my large, shallow plastic bowl).\nAdd the starch and salt. Crack in an egg.\nMix well with a spoon or a whisk.\nHeat the oil in a large, deep pan over medium-high heat until it reaches about 340° to 350° F. If you don’t have a kitchen thermometer, test the oil temperature by dipping a tip of a chicken piece into it. If it bubbles, it’s ready.\nDip a chicken piece into the batter and wipe off the excess with the side of the bowl. Add it to the oil one piece at a time, working in batches if your pan is small. My 12 inch large pan is perfect for frying 4 pieces of chicken thighs at a time. Repeat with the rest of the chicken pieces until all are in the pan.\nDeep-fry, turning the chicken with tongs, until all sides are light golden brown and crunchy, about 5 to 6 minutes. As each piece is done, transfer it to a strainer over a bowl to drain the excess oil.\nReheat the oil for about 1 minute and add all the chicken again. It will look a little soggy at first. Deep-fry, turning occasionally, until all the chicken pieces are dark golden brown and very crunchy, another 5 to 6 minutes.\nTransfer the chicken pieces to the strainer over the bowl to drain. Let them cool for a few minutes until you can handle them.\nAdd the sliced chili pepper to the sauce and mix it together.\nDrain the soaking green onion and onion through a strainer set in your sink. Rinse it under cold running water, turning it over by hand for 10 seconds. Drain well.\nCut the chicken into bite size pieces and put them on a large plate lined with the lettuce. Spoon the peppers out of the sauce and put them on top of the chicken.\nAdd the shredded green onion and onion and gently pour the rest of the sauce all over top.\nServe right away.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("Korean", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Today I'm introducing you to another Korean Chinese dish called yuringi (pronounced yu-rin-gi). It's fried chicken with peppers and a light sweet sour sauce. A piece of crunchy chicken soaked in sweet, sour, salty sauce tastes so crispy and juicy! The chopped spicy fresh pepper served with this...",
            self.harvester_class.description(),
        )



from recipe_scrapers.plugins import (
    ExceptionHandlingPlugin,
    HTMLTagStripperPlugin,
    NormalizeStringPlugin,
    OpenGraphImageFetchPlugin,
    SchemaOrgFillPlugin,
)

# Plugins to be attached.
# The upper most plugin is the "outer most" executed.
# Check recipe_scrapers.settings.template.py for ways to extend.
PLUGINS = (
    ExceptionHandlingPlugin,
    HTMLTagStripperPlugin,
    NormalizeStringPlugin,
    OpenGraphImageFetchPlugin,
    SchemaOrgFillPlugin,
)

SUPPRESS_EXCEPTIONS = False
# Applicable only if SUPPRESS_EXCEPTIONS is True, otherwise ignored
# silence <anyScraper>.[method]() exception and return the value
# as listed in the config here.
ON_EXCEPTION_RETURN_VALUES = {
    "title": None,
    "total_time": None,
    "yields": None,
    "image": None,
    "ingredients": None,
    "instructions": None,
    "instructions_list": None,
    "ratings": None,
    "reviews": None,
    "links": None,
    "language": None,
    "nutrients": None,
}


# logging.DEBUG     # 10
# logging.INFO      # 20
# logging.WARNING   # 30
# logging.ERROR     # 40
# logging.CRITICAL  # 50
# https://docs.python.org/3/howto/logging.html
LOG_LEVEL = 30



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class ClosetCooking(AbstractScraper):
    @classmethod
    def host(cls):
        return "closetcooking.com"

    def title(self):
        return normalize_string(
            self.soup.find("h1", {"class": "entry-title"}).get_text()
        )

    def total_time(self):
        return get_minutes(self.soup.find(itemprop="totalTime").parent)

    def yields(self):
        return get_yields(self.soup.find(itemprop="recipeYield").parent)

    def ingredients(self):
        ingredients = self.soup.findAll("li", {"itemprop": "recipeIngredient"})

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = self.soup.findAll("li", {"itemprop": "recipeInstructions"})

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )



from responses import GET

from recipe_scrapers.foodnetwork import FoodNetwork
from tests import ScraperTest


class TestFoodNetworkScraper(ScraperTest):

    scraper_class = FoodNetwork

    @classmethod
    def expected_requests(cls):
        yield GET, "https://www.foodnetwork.com/recipes/tyler-florence/chicken-marsala-recipe-1951778", "tests/test_data/foodnetwork.testhtml"

    def test_host(self):
        self.assertEqual("foodnetwork.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://foodnetwork.co.uk/recipes/chicken-marsala/?utm_source=foodnetwork.com&utm_medium=domestic",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Chicken Marsala")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Food Network")

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://d2v9mhsiek5lbq.cloudfront.net/eyJidWNrZXQiOiJsb21hLW1lZGlhLXVrIiwia2V5IjoiZm9vZG5ldHdvcmstaW1hZ2UtZGVmYXVsdC1wbGFjZWhvbGRlci5qcGciLCJlZGl0cyI6eyJyZXNpemUiOnsiZml0IjoiY292ZXIiLCJ3aWR0aCI6MTkyMCwiaGVpZ2h0IjoxMDgwfSwianBlZyI6eyJxdWFsaXR5Ijo3NSwicHJvZ3Jlc3NpdmUiOnRydWV9fX0=",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 (225g) boneless skinless chicken breasts",
                "Plain flour, for dredging, plus 2 tbsp",
                "85g butter",
                "1 tbsp olive oil",
                "80g sliced mushrooms",
                "2 tbsp minced garlic",
                "4 tbsp Marsala wine",
                "500ml beef stock",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "1) Put the chicken breasts between 2 pieces of waxed paper and flatten with a meat pounder until thin. Cut each chicken breast into 4 pieces.\n\n2) Add some flour to a shallow bowl. Dredge the chicken in the flour and shake off the excess flour.\n\n3) Add the butter and olive oil to a large saute pan over high heat and heat until it sizzles, do NOT let it brown. Add the chicken and saute until brown on both sides.\n\n4) Stir in the sliced mushrooms and saute briefly, then add the garlic. Add the Marsala and simmer for 3 minutes, then stir in the remaining 2 tbsp of flour.\n\n5) Pour in the beef stock and leave to simmer until the sauce thickens, about 3 to 5 minutes. Transfer the chicken to a serving platter and serve.\n\nThis recipe was provided by professional chefs and has been scaled down from a bulk recipe provided by a restaurant. The Food Network kitchens chefs have not tested this recipe, in the proportions indicated, and therefore we cannot make any representation as to the results.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.zeitwochenmarkt import ZeitWochenmarkt
from tests import ScraperTest

# test recipe's URL
# https://www.zeit.de/zeit-magazin/wochenmarkt/2021-08/kohlrabi-fenchel-carpaccio-fior-di-latte-rezept


class TestZeitWochenmarktScraper(ScraperTest):

    scraper_class = ZeitWochenmarkt

    def test_host(self):
        self.assertEqual("zeit.de", self.harvester_class.host())

    def test_title(self):
        self.assertEqual("Kohlrabi-Fenchel-Carpaccio", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Angelika Schwaff")

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_language(self):
        self.assertEqual("de", self.harvester_class.language())

    def test_image(self):
        self.assertEqual(
            "https://img.zeit.de/zeit-magazin/wochenmarkt/2021-08/kohlrabi-fenchel-carpaccio/wide__1300x731",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                '1 Kugel "Fior di Latte" (oder eine Burrata)',
                "1 Kohlrabi (klein)",
                "1 Fenchelknolle",
                "1/2 Bund Basilikum",
                "2 EL Pistazienkerne",
                "4 EL Olivenöl",
                "2 EL Zitronensaft",
                "1 TL Honig",
                "Salz",
                "Pfeffer",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        instructions = """Den Kohlrabi schälen, den Fenchel waschen. Beide vom unteren Strunk befreien. Den Kohlrabi einmal von der Spitze zum Boden in der Mitte durchschneiden. Nun beide Gemüse mit einem scharfen und stabilen Messer hauchdünn schneiden, bessere Ergebnisse erzielt man auf einer Mandoline. Idealerweise sind die Scheiben so dünn, dass man fast hindurchschauen kann.
Nun aus dem Olivenöl, dem Zitronensaft und Honig ein Dressing zusammenrühren, mit Salz und Pfeffer abschmecken. Die dünnen Gemüsescheiben fächerartig auf einen großen Teller legen, den Käse zerreißen und darauf verteilen. Das Dressing darübergeben und mit Basilikum und Pistazien servieren."""
        self.assertEqual(instructions, self.harvester_class.instructions())



from recipe_scrapers.nytimes import NYTimes
from tests import ScraperTest


class TestNYTimesScraper(ScraperTest):

    scraper_class = NYTimes

    def test_host(self):
        self.assertEqual("cooking.nytimes.com", self.harvester_class.host())

    def test_image(self):
        self.assertEqual(
            "https://static01.nyt.com/images/2020/05/28/dining/lp-cacio-e-pepe-crackers/merlin_172657737_693784a8-529d-4496-9e60-3ff2af3c7735-articleLarge.jpg",
            self.harvester_class.image(),
        )

    def test_canonical_url(self):
        self.assertEqual(
            "https://cooking.nytimes.com/recipes/1021128-cacio-e-pepe-crackers",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Cacio e Pepe Crackers")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Laurie Ellen Pellicano")

    def test_ratings(self):
        self.assertEqual(self.harvester_class.ratings(), 4.0)

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("160 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 1/2 cups/190 grams unbleached all-purpose flour (see Tip)",
                "1 tablespoon freshly ground black pepper, plus more for finishing",
                "1/2 teaspoon kosher salt",
                "1/2 teaspoon onion powder (optional)",
                "1/2 teaspoon ground mustard (optional)",
                "1/8 teaspoon garlic powder (optional)",
                "5 ounces/145 grams white Cheddar, roughly grated (about 1 1/4 cups packed)",
                "3 ounces/85 grams Asiago cheese, roughly grated (about 3/4 cup)",
                "5 tablespoons/70 grams unsalted butter, cold and cubed",
                "1/4 cup/25 grams finely ground Pecorino Romano cheese (or Parmigiano-Reggiano or more Asiago), for sprinkling",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In the bowl of a food processor, add the flour, pepper, salt and spices (if using), and pulse to combine.\nAdd the Cheddar, Asiago and butter, and pulse several times, then let the mixer run until the dough comes mostly together around the blade, 1 to 3 minutes. It’s OK if the dough is a little pebbly, but it should clump easily when you squeeze it. (You can also prepare this dough by hand, though you’ll need to bring the butter to room temperature first. Mix all your dry ingredients in a medium bowl. Then, in a large bowl, mix Cheddar, Asiago and butter to form a paste. Add the flour mixture and knead the dough together.)\nPull your dough out of your bowl onto a flat surface and gently knead it into a smooth ball. Split your dough in half and shape each half into a rectangle. Using a rolling pin, roll each piece until about 1/2-inch thick, dusting a tiny bit of flour on your pin, if needed, to prevent the dough from sticking. (If you don’t want to bake all the crackers now, you can freeze dough in 1/2-inch-thick blocks.)\nPlace a piece of dough in the center of an 18-inch-long piece of parchment paper. Roll the dough on the parchment paper, working from the center outward. (You want the dough to adhere to the bottom layer of parchment, but if your rolling pin sticks to the surface, lightly dust it with flour.) When your dough is about 1/4-inch thick, lay another piece of parchment, plastic wrap, or a silicone baking mat over the surface of your dough. Continue to roll the dough out 1/8- to 1/16-inch thick, as thin as your arms will allow, pressing together any cracks that may form. (You can also use an etching motion, moving your pin from the center out toward the edges across your dough.) Rotate the parchment in front of you with every few strokes to ensure you are rolling the dough evenly.\nPeel back the top layer of parchment and sprinkle the surface with half the Pecorino Romano and a dozen or so grinds of black pepper across the surface. Lightly roll over once more with your rolling pin so the cheese and pepper adheres to the cracker dough. Transfer this sheeted dough onto a baking sheet and chill in the fridge or freezer until firm, about 15 minutes. (If you let it chill longer, just pull it out and let it temper a bit before proceeding.) Repeat with the second piece of dough.\nWhen the dough is nearly chilled, arrange the racks in the upper and lower third of the oven and heat to 325 degrees. Remove one sheet of dough from the tray and place on a work surface.\nUsing a pastry wheel (fluted is nice), pizza cutter or a sharp knife and a ruler, cut 1-inch squares across the surface of the dough. (A 1-inch-thick ruler or tracer made from card stock or cardboard comes in handy here.) Transfer crackers to parchment-lined baking sheets with 1/2-inch space in between. (They will not spread much.) If your dough warms up or is difficult to peel and place, just slip it back into the freezer still attached to your parchment paper and let it firm up, then proceed.\nBake the crackers in the center of your oven for 14 to 20 minutes (depending on thickness), rotating trays midway through baking to ensure they color evenly. Crackers will be just golden at the edges and the surface should be firm to the touch. You want them to dry crisp. (Test by pulling one cracker off the tray, let it quickly cool and break it in half to see how it snaps.) Remove from the oven and cool on trays.\nOnce fully cooled, store crackers in a tin or covered container for up to 4 weeks.",
            self.harvester_class.instructions(),
        )


# https://cooking.nytimes.com/recipes/1021128-cacio-e-pepe-crackers



import responses

from recipe_scrapers.marleyspoon import MarleySpoon
from tests import ScraperTest


class TestMarleySpoonScraper(ScraperTest):

    scraper_class = MarleySpoon

    @classmethod
    def expected_requests(cls):
        yield responses.GET, "https://marleyspoon.de/menu/113813-glasierte-veggie-burger-mit-roestkartoffeln-und-apfel-gurken-salat", "tests/test_data/marleyspoon.testhtml"
        yield responses.GET, "https://api.marleyspoon.com/recipes/113813?brand=ms&country=de&product_type=web", "tests/test_data/marleyspoon.testjson"

    def test__get_json_params(self):
        self.assertEqual(
            (
                "https://api.marleyspoon.com/recipes/113813?brand=ms&country=de&product_type=web",
                "Bearer eyJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJtcyIsImNvdW50cnkiOiJkZSIsImJyYW5kIjoibXMiLCJ0cyI6MTY1Mzg4ODg3NiwicmFuZG9tX2lkIjoiMGY4YjZkIn0.quv6_xQk0EjwKmHn7u_CltqMkPuNen-N6kncGHTjcbg",
            ),
            self.harvester_class._get_json_params(),
        )

    def test_host(self):
        self.assertEqual("marleyspoon.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Glasierte Veggie-Burger mit Röstkartoffeln und Apfel-Gurken-Salat",
        )

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://marleyspoon.com/media/recipes/113813/main_photos/large/veggie_burger_mit_ahornsirupglasur-52496098c98adf7b4a82b8b38fdf14dc.jpeg",
            self.harvester_class.image(),
        )

    def test_nutrients(self):
        self.assertEqual(
            {
                "calories": "1143kcal",
                "carbs": "103.8g",
                "fat": "68.0g",
                "proteins": "27.8g",
            },
            self.harvester_class.nutrients(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "25g Erdnüsse, geröstet & gesalzen",
                "2 vegane Burgerpattys",
                "2 Burgerbrötchen mit Sesam",
                "1 Päckchen Sriracha-Sauce",
                "25ml Sojasauce",
                "2 Stangensellerie",
                "1 Minigurke",
                "1 Apfel",
                "1 Packung festkochende Kartoffeln",
                "2EL Mayonnaise",
                "2½TL Honig",
                "½TL Senf",
                "Salz",
                "Zucker",
                "Olivenöl",
                "Essig",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Den Backofen auf 240°C (220°C Umluft) vorheizen. Die Kartoffeln samt Schale in ca. 1cm breite Spalten schneiden und auf einem mit Backpapier ausgelegten Backblech mit 1EL Olivenöl sowie 1/2TL Salz vermengen. Die Kartoffeln im Ofen auf mittlerer Schiene 20–25Min. goldbraun rösten, zwischendurch einmal wenden.\n2EL Olivenöl, 1EL hellen Essig, 1/2TL Senf, 1/2TL Honig und 1 kräftige Prise Salz zu einem Dressing verrühren. Den Apfel vierteln, entkernen und in ca. 1cm große Würfel schneiden. Die 1/2 der Gurke ebenfalls in ca. 1cm große Würfel schneiden. Den Sellerie in dünne Scheiben schneiden. Die Apfel- und Gurkenwürfel und den Sellerie mit dem Dressing vermengen.\nDie restliche Gurke mit einem Sparschäler in breite Streifen schneiden. 1EL hellen Essig, 1TL Zucker und 1 kräftigen Prise Salz verrühren, bis sich Zucker und Salz aufgelöst haben. Mit den Gurkenstreifen vermengen und beiseitestellen.\nDie Sojasauce mit 2TL Honig und 1EL hellem Essig verrühren. Getrennt davon die Sriracha-Sauce mit 2EL Mayonnaise verrühren. Tipp: Wenn Kinder mitessen, ggf. weniger Sriracha-Sauce verwenden oder diese separat servieren. Die Burgerbrötchen auf einem Backrost 2–3Min. im Ofen aufbacken.\nDie Pattys in einer mittelgroßen Pfanne mit 1EL Olivenöl bei mittlerer Hitze auf jeder Seite 1–2Min. goldbraun braten. Die Soja-Würzsauce dazugeben, 1–2Min. köcheln lassen und die Pattys in der Glasur wenden, sodass sie vollständig damit bedeckt sind.\nDie Brötchen aufschneiden, mit dem Dip bestreichen und nach Geschmack mit den Pattys, den Gurkenstreifen und je ca. 1EL Apfel-Gurken-Salat belegen. Den restlichen Salat mit den Erdnüssen bestreuen und zu den Burgern, den Röstkartoffeln und ggf. übrigen Gurkenstreifen servieren. Ggf. übrigen Dip oder mehr Mayonnaise nach Geschmack dazu reichen.",
            self.harvester_class.instructions(),
        )

    def test_author(self):
        self.assertEqual("Lieke", self.harvester_class.author())

    def test_description(self):
        self.assertEqual(
            "Heute gibt’s Burgergenuss auf hohem Niveau: Die saftigen Pattys werden in einer würzig-süßen Glasur gebraten, ehe sie mit eingelegten Gurkenstreifen, pikantem Sriracha-Dip und knackigem Salat mit frisch-fruchtigem Apfel und würzigem Sellerie das Burgerbrötchen zieren. Der Salat fungiert übrigens – schick mit Erdnüssen bestreut – neben knusprigen Ofenkartoffeln auch noch zusätzlich als Beilage. Lass es dir schmecken!",
            self.harvester_class.description(),
        )

    def test_links(self):
        expected = "https://marleyspoon.com/media/pdf/recipe_cards/113813/R-113813_-_Glasierte_Veggie-Burger-CV.pdf"
        self.assertTrue(
            any(link["href"] == expected for link in self.harvester_class.links())
        )

    def test_language(self):
        self.assertEqual("de-DE", self.harvester_class.language())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_yields, normalize_string


class CookingLight(AbstractScraper):
    @classmethod
    def host(cls):
        return "cookinglight.com"

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return get_yields(self.schema.yields())

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredients = self.soup.find("div", {"class": "ingredients"}).ul.findAll("li")
        return "\n".join(
            [normalize_string(ingredient.get_text()) for ingredient in ingredients]
        )

    def instructions(self):
        instructions = self.soup.find("div", {"class": "recipe-instructions"}).findAll(
            "div", {"class": "step"}
        )
        return "\n".join([normalize_string(instr.get_text()) for instr in instructions])

    def ratings(self):
        try:
            return self.schema.ratings()
        except Exception:
            return None



SUPPRESS_EXCEPTIONS = True


# the most powerful feature is adding custom plugins
# for example to add "inner-most" plugin:
# PLUGINS += (
#     "path.to.my.custom_plugin"
# )
#
# and to add "outer-most" plugin:
# PLUGINS = (
#     "path.to.my.custom_plugin",
# ) + PLUGINS



from recipe_scrapers.downshiftology import Downshiftology
from tests import ScraperTest


class TestDownshiftologyScraper(ScraperTest):

    scraper_class = Downshiftology

    def test_host(self):
        self.assertEqual("downshiftology.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Lisa Bryan", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Greek Chicken Kabobs", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(55, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://i2.wp.com/www.downshiftology.com/wp-content/uploads/2020/09/Greek-Chicken-Kabobs-3.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1/4 cup olive oil",
                "2 tablespoons red wine vinegar",
                "3 tablespoons lemon juice",
                "1 teaspoon Dijon mustard",
                "3 garlic cloves (minced)",
                "1 teaspoon dried oregano",
                "1/2 teaspoon salt",
                "1/4 teaspoon black pepper",
                "1 1/2 pounds boneless skinless chicken breasts (about 3 large chicken breasts, cut into 1 1/2-inch pieces.)",
                "1 red bell pepper (seeded, cut into 1 1/2-Inch pieces)",
                "1 yellow bell pepper (seeded, cut into 1 1/2-inch pieces)",
                "1 red onion (cut into 1 1/2-inch chunks)",
                "1 zucchini (sliced)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Make marinade\nTo make the marinade, whisk together the olive oil, red wine vinegar, lemon juice, Dijon mustard, minced garlic, dried oregano, salt, and pepper.\nMarinate chicken\nPlace chicken pieces in a glass dish and pour the marinade over the chicken. Cover and marinate in the fridge for at least one hour.\nThread skewers\nLight a gas or charcoal grill on medium-high heat. Thread the skewers with pieces of red onion, chicken, zucchini, and bell pepper. You can alternate the order.\nGrill kabobs\nPlace the kabobs on the preheated grill, and cook about 5-7 minutes per side. The kabobs are done when the chicken is cooked through and the vegetables are lightly charred, about 15 minutes.\nServe with lemon wedges and tzatziki sauce.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



from recipe_scrapers.copykat import CopyKat
from tests import ScraperTest


class TestCopyKat(ScraperTest):

    scraper_class = CopyKat

    def test_host(self):
        self.assertEqual("copykat.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://copykat.com/make-tender-beef-tips-in-gravy/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Make Tender Beef Tips and Gravy"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Stephanie Manley")

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("10 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://copykat.com/wp-content/uploads/2020/11/Beef-Tips-and-Gravy-Pin2.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 cup flour",
                "1 teaspoon salt",
                "1/4 teaspoon black pepper",
                "3 pounds lean chuck roast",
                "48 ounces beef stock",
                "1 tablespoon vegetable oil",
                "2 teaspoons gravy master",
                "1 cup onions (chopped)",
                "16 ounces noodles (or rice for serving)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In a small bowl add flour, salt, and pepper. Stir the salt and pepper into the flour. Cut and trim roast into small bite-sized pieces. Dredge beef pieces in seasoned flour shake off excess flour.\nInstant Pot Directions\nSet the Instant Pot to saute, add oil. When the oil has heated drop in several pieces of the beef. Cook seasoned beef on all sides until lightly browned. Cook beef in small batches. When all of the beef is cooked add it back to the Instant Pot.\nAdd 1 cup of onion, two teaspoons of Gravy Master, and beef stock. Place lid on high and cook for 15 minutes on high pressure. Release pot after cooking with either a quick release or a natural release.\nSlow Cooker Directions\nBrown the beef in a large skillet in small batches with some vegetable oil. Add the browned beef, beef broth, onion, and Gravy Master to the slow cooker. Cook for 4 to 6 hours on low.\nIf the liquid hasn't thickened up to your desired consistency, you can thicken it up by mixing 1 tablespoon of butter and one tablespoon of flour together. Stir this into the liquid and it will thicken up the gravy in the slow cooker.\nServing\nPrepare noodles or rice according to package instructions.\nServe beef tips and gravy over noodles or rice.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5, self.harvester_class.ratings())

    def test_description(self):
        self.assertEqual(
            "You can make amazingly tender beef tips and gravy in an Instant Pot or slow cooker.",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class Cookstr(AbstractScraper):
    @classmethod
    def host(cls):
        return "cookstr.com"

    def title(self):
        return normalize_string(
            self.soup.find("h1", {"class": "articleHeadline"}).get_text()
        )

    def total_time(self):
        sections = self.soup.findAll("div", {"class": "articleAttrSection"})
        total_time = 0
        for section in sections:
            time = section.find(string="Total Time")
            if time:
                total_time += get_minutes(time.parent.parent)
        return total_time

    def yields(self):
        sections = self.soup.findAll("span", {"class": "attrLabel"})
        for section in sections:
            serves = section.find(string="Serves")
            if serves:
                return get_yields(serves.parent.parent)
        raise Exception("Servings amount not found")

    def ingredients(self):
        ingredients = self.soup.find("div", {"class": "recipeIngredients"})

        return [
            normalize_string(ingredient.get_text())
            for ingredient in ingredients.findAll("li")
        ]

    def instructions(self):
        instructions = self.soup.find("div", {"class": "stepByStepInstructionsDiv"})

        return "\n".join(
            [
                normalize_string(instruction.get_text())
                for instruction in instructions.findAll("p")
            ]
        )



from recipe_scrapers.marthastewart import MarthaStewart
from tests import ScraperTest


class TestMarthaStewart(ScraperTest):

    scraper_class = MarthaStewart

    def test_host(self):
        self.assertEqual("marthastewart.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.marthastewart.com/336792/breaded-chicken-breasts",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Breaded Chicken Breasts")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Martha Stewart")

    def test_total_time(self):
        self.assertEqual(25, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://imagesvc.meredithcorp.io/v3/mm/image?url=https%3A%2F%2Fassets.marthastewart.com%2Fstyles%2Fwmax-750%2Fd31%2Fbreaded-chicken-cutlets-d104370%2Fbreaded-chicken-cutlets-d104370_horiz.jpg%3Fitok%3DdnK5TccB",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertSetEqual(
            set(
                [
                    "3 large eggs",
                    "Coarse salt",
                    "1/3 cup all-purpose flour",
                    "3 1/2 cups fresh breadcrumbs",
                    "1 cup vegetable oil",
                    "8 thin chicken cutlets (about 1 1/2 pounds total)",
                    "Lemon wedges, for serving (optional)",
                ]
            ),
            set(self.harvester_class.ingredients()),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "In a shallow dish, whisk eggs with teaspoon salt; let stand 5 minutes. In another shallow dish, season flour with 1/4 teaspoon salt. In a third shallow dish, season breadcrumbs with 1 teaspoon salt.",
                    "In a large cast-iron skillet or other heavy deep skillet, heat oil over medium. Meanwhile, pat chicken dry with paper towels. Coat in flour, shaking off excess, then dip in egg (letting excess drip off). Dredge in breadcrumbs, turning twice and patting to adhere.",
                    "Increase heat to medium-high. Working in batches, add chicken to skillet; cook, gently shaking skillet occasionally, until chicken is browned, about 4 minutes. Turn with tongs; cook until browned and opaque throughout, 2 to 3 minutes more (if browning too quickly, lower heat). Between batches, skim off brown crumbs from oil with a slotted spoon. Drain chicken on paper towels; season with salt.",
                ]
            ),
            self.harvester_class.instructions(),
        )



from recipe_scrapers.koket import Koket
from tests import ScraperTest


class TestKoketScraper(ScraperTest):

    scraper_class = Koket

    def test_host(self):
        self.assertEqual("koket.se", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Tommy Myllymäki", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Myllymäkis toast skagen", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(25, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://img.koket.se/standard-mega/myllymakis-toast-skagen-2.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 kg räkor med skal (gärna färska av fin kvalitet)",
                "2 äggulor",
                "2 tsk senap",
                "1 msk vitvinsvinäger",
                "6 dl matolja",
                "1 kruka dill",
                "10 cm färsk pepparrot, skalad",
                "4 skiva vitt bröd (ej levain)",
                "smör, till stekning",
                "50 g löjrom",
                "1 citron",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Skala alla räkor och ställ åt sidan.\nGör en majonnäs genom att lägga ner äggulor, senapen och vinägern i en bunke. Tillsätt matoljan i en tunn stråle medan du vispar hela tiden. Använd elvisp eller handvisp. När majonnäsen är tjock och du ser dragen/spåren av vispen i majonnäsen är den klar.\nLägg alla räkor i en bunke, tillsätt fint plockad dill och blanda ner lite majonnäs i taget.\nTillsätt lite riven pepparrot och smaka av. Slå på mer majonnäs för en rinnigare röra eller mer pepparrot för mer sting.\nTa fram brödet och skär ut önskad form utan att ta med kanterna, använd en skål eller ett glas som mall om ni vill ha runda bröd. Stek sedan gyllene i smör.\nLägg upp bröden på tallrik, toppa med skagenröra och en rejäl klick löjrom. Avsluta med en dillkvist och en citronskiva.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.6, self.harvester_class.ratings())



from recipe_scrapers.nhshealthierfamilies import NHSHealthierFamilies
from tests import ScraperTest


class TestNHSHealthierFamiliesScraper(ScraperTest):

    scraper_class = NHSHealthierFamilies

    def test_host(self):
        self.assertEqual("nhs.uk", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("NHS Better Health", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Chilli con carne", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://assets.nhs.uk/campaigns-cms-prod/images/Chilli-con-carne_x7m8d91.width-320.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "300g extra-lean minced beef",
                "1 large onion, finely chopped",
                "2 garlic cloves, finely chopped",
                "400g chopped tomatoes",
                "2 tablespoons tomato purée",
                "2 teaspoons chilli powder",
                "1 teaspoon ground cumin",
                "1 red pepper, deseeded and chopped",
                "2 handfuls of cup or button mushrooms, sliced",
                "410g red kidney beans, drained",
                "150ml reduced-salt vegetable or chicken stock",
                "300g easy-cook white or brown rice",
                "1 pinch ground black pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Heat a large saucepan and add the minced beef, a handful at a time, cooking it until browned. Add the onion and garlic, then cook for another 2 to 3 minutes.",
                    "Add the chopped tomatoes, tomato purée, spices, red pepper, mushrooms, kidney beans and stock. Stir well, bring to the boil, then lower the heat and simmer gently for 15 to 20 minutes.",
                    "Meanwhile, cook the rice according to pack instructions.",
                    "Season the chilli with pepper and serve with the boiled rice.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "This classic chilli is packed with flavour. It also freezes well, so is perfect for batch-cooking.",
            self.harvester_class.description(),
        )



from recipe_scrapers.eatwhattonight import EatWhatTonight
from tests import ScraperTest


class TestEatWhatTonight(ScraperTest):

    scraper_class = EatWhatTonight

    def test_host(self):
        self.assertEqual("eatwhattonight.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Eat What Tonight", self.harvester_class.author())

    def test_canonical_url(self):
        self.assertEqual(
            "http://eatwhattonight.com/2020/08/ginger-soya-chicken/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Ginger Soya Chicken")

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "http://eatwhattonight.com/wp-content/uploads/2020/08/DSC_0317-scaled.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "180g boneless chicken, diced (marinate with 2 tsp light soya sauce, 1/2 tsp sugar and pinch of pepper for 30mins)⁣",
                "1 thumb sized ginger, thinly sliced⁣",
                "Few sprigs of spring onion⁣",
                "1/2 tsp minced garlic⁣",
                "1 tbsp cooking oil⁣",
                "2 tsp shaoxing wine⁣",
                "1 tbsp oyster sauce⁣",
                "2 tbsp dark soya sauce⁣",
                "125ml water⁣",
                "1 tsp sugar⁣",
                "1 tsp sesame oil⁣",
                "2 tsp corn flour⁣",
                "2 tbsp water⁣",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Heat up oil and saute garlics, white parts of spring onion and ginger till fragrant.⁣\nAdd in marinated chicken with the light soya sauce and mix well.⁣\nAdd in water.\nFollowed by the rest of the seasoning and allow the sauce to cook on high heat.⁣\nAdd in green parts of spring onions.\nAdd in sauce thickening.\nWhen sauce has thickened, add in shaoxing wine and remove from heat.⁣Serve hot immediately with rice.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.popsugar import PopSugar
from tests import ScraperTest


class TestPopSugarScraper(ScraperTest):

    scraper_class = PopSugar

    def test_host(self):
        self.assertEqual("popsugar.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual("Rainbow Pasta", self.harvester_class.title())

    def test_image(self):
        self.assertEqual(
            "https://media1.popsugar-assets.com/files/thumbor/QpXN5ex3L8WRyMBIBlypdBwKX-Q/fit-in/1024x1024/filters:format_auto-!!-:strip_icc-!!-/2016/09/02/018/n/1922195/f31877d9_5cdf801a_Rainbow_Pasta_HERO/i/Rainbow-Pasta-Food-Video.jpg",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(49, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 tablespoons grated parmesan cheese",
                "Non-stick cooking spray",
                "4 baby beetroot bulbs, quartered",
                "1 2-inch piece of turmeric, grated",
                "1 red cabbage, roughly chopped",
                "1 teaspoon baking soda",
                "Water",
                "1 pound dry spaghetti, cooked 2 minutes below the recommended cooking time",
                "6 tablespoons unsalted butter",
                "3 garlic cloves, smashed",
                "1 shallot, roughly chopped",
                "2 tablespoons fresh chives, roughly chopped",
                "3 sprigs fresh thyme",
                "1 teaspoon lemon zest",
                "2 tablespoons lemon juice",
                "Kosher salt to taste",
                "Parmesan cheese, for garnish",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "For parmesan tuiles: Arrange the parmesan cheese into 4 piles on a greased 10-inch skillet. Turn the heat to medium and allow the cheese to melt and turn golden brown around the edges, about 4-6 minutes. Remove the pan from the heat and allow the parmesan tuiles to cool in the pan before transferring to a paper towel lined plate. Repeat with more cheese if desired.\nFor pasta: Add the beets to the bowl of a food processor and pulse about 10 times until coarsely chopped. Add the beet mixture to a small saucepan and add enough water to just cover the beets (about 2 cups). Simmer on medium for about 5 minutes until the water becomes a deep and vibrant pink color. Strain the liquid through a cheesecloth lined strainer into a small bowl and set aside to cool. Discard or store any leftover beet pulp.\nIn a clean saucepan, add the turmeric and 3 cups of water. Bring to a simmer for 5 minutes, strain through a cheesecloth-lined strainer into a small bowl and set aside to cool. The mixture will look slightly yellow/orange.\nLastly, add the cabbage to a large, heavy-bottom stock pot and just barely cover with water (about 5 cups). Bring the water to a boil over high heat, then lower the temperature to medium-low and simmer for about 8 minutes until the water becomes a deep purple color. Strain the liquid through a cheesecloth-lined strainer, evenly into 2 small bowls and set aside to cool. Add the baking soda to one of the bowls containing the cabbage liquid and delicately stir. The liquid will turn from purple to vibrant blue right before your eyes.\nIn another small bowl, combine equal parts blue and yellow dye to create a green shade. You should now have 5 natural food colors to dye the spaghetti with.\nDivide cooked pasta into 5 1-gallon resealable bags. Add each dye to the bag of spaghetti you'd like to color. The dye should just cover the pasta. Let sit for at least 20 minutes or up to 2 hours. Drain each batch of dyed pasta on a paper towel-lined plate or sheet tray and set aside.\nFor the sauce: In a small saucepan over medium-high heat, combine butter, garlic, shallot, chives, and thyme. Bring the butter to a boil and reduce the heat to medium-low. Simmer the aromatic herbs for about 6 minutes or until fragrant.\nRemove the butter from the heat and add lemon zest, lemon juice, and salt to taste. Strain the butter into a small bowl through a cheesecloth-lined strainer and toss with the rainbow pasta!\nGarnish your colorful creation with parmesan cheese and of course, the parmesan tuile you made earlier.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.closetcooking import ClosetCooking
from tests import ScraperTest


class TestClosetCooking(ScraperTest):

    scraper_class = ClosetCooking

    def test_host(self):
        self.assertEqual("closetcooking.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.closetcooking.com/jalapeno-popper-skillet-chicken/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Jalapeno Popper Skillet Chicken"
        )

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("5 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 tablespoon oil",
                "1 pound chicken, boneless and skinless, diced",
                "salt and pepper to taste",
                "1 small onion, diced",
                "2 jalapenos, sliced or diced",
                "2 cloves garlic, chopped",
                "1 cup chicken broth",
                "4 ounces cream cheese, softened",
                "1 cup cheddar cheese, shredded",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Heat the oil in a pan over medium-high heat, add the chicken (seasoned with salt and pepper) and cook until lightly golden brown.\nAdd the onions and jalapenos and cook until tender, about a minute before adding the garlic and cooking another minute.\nAdd the chicken broth and deglaze the skillet by scraping the brown bits up off of the bottom of the pan as the broth sizzles.\nAdd the cheese and cook until it has melted and the sauce is nice and smooth",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.motherthyme import MotherThyme
from tests import ScraperTest


class TestMotherThymeScraper(ScraperTest):

    scraper_class = MotherThyme

    def test_host(self):
        self.assertEqual("motherthyme.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "http://www.motherthyme.com/2014/01/cinnamon-roll-oatmeal.html",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Cinnamon Roll Oatmeal")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Jenn (Mother Thyme)")

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 3/4 cup milk",
                "2 tablespoons brown sugar",
                "1 tablespoon sugar",
                "1/2 teaspoon vanilla extract",
                "1/4 teaspoon salt",
                "1 1/2 cups old-fashioned rolled oats",
                "Topping",
                "2 tablespoons butter (melted)",
                "3 tablespoons packed light brown sugar",
                "1 1/2 teaspoons cinnamon",
                "Glaze",
                "1 ounce cream cheese (softened)",
                "1 tablespoon milk",
                "1/8 teaspoon vanilla extract",
                "5 tablespoons confectioners' sugar",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Oatmeal\nIn a large saucepan add milk, sugars, vanilla and salt and bring to a boil over medium heat.\nStir in oats, return to a boil and continue to cook, stirring occasionally for 3-5 minutes until oatmeal begins to thicken.\nCover and remove from heat. Let sit for about 3 minutes.\nTopping\nIn a small bowl mix butter, brown sugar and cinnamon until combined.\nGlaze\nMicrowave cream cheese in a small bowl for about 10 seconds until just melted.\nStir in confectioners' sugar until combined.\nStir in milk and vanilla until creamy.\nNote: Add in a little more confectioners' sugar if glaze is too thin or a little more milk if glaze is too thick until desired consistency.\nAssemble\nPlace desired amount of oatmeal in serving bowl. Drizzle with some of the cinnamon sugar topping and then drizzle on top of that some of the cream cheese glaze.\nServe warm.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.91, self.harvester_class.ratings())



from recipe_scrapers.bakingsense import BakingSense
from tests import ScraperTest


class TestBakingSense(ScraperTest):

    scraper_class = BakingSense

    def test_host(self):
        self.assertEqual("baking-sense.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.baking-sense.com/2020/05/13/sourdough-bundt-cake/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Sourdough Bundt Cake with Buttermilk Glaze"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Eileen Gray")

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("16 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.baking-sense.com/wp-content/uploads/2020/05/sourdough-bundt-9a.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 large eggs (room temperature)",
                "2 large yolks (room temperature)",
                "1 tablespoon vanilla extract",
                "8 oz sourdough discard (1 cup, room temperature)",
                "9 oz cake flour (2 cups)",
                "11 oz granulated sugar (1 1/3 cups)",
                "2 teaspoons baking powder",
                "1/2 teaspoon table salt",
                '6 oz unsalted butter (room temperature, cut into 1" chunks)',
                "4 oz buttermilk (1/2 cup, room temperature)",
                "8 oz confectioner's sugar (2 cups)",
                "1 teaspoon vanilla",
                "2 oz buttermilk (1/4 cup)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 350°F. Generously butter and flour a 12 cup Bundt pan.\nMake the batter\nWhisk together the eggs, yolks, vanilla and the discard, set aside.\nSift the flour, sugar, baking powder and salt into a mixer bowl. Mix on low speed to combine the dry ingredients. With the mixer running, toss the chunks of butter into the flour mixture.\nAdd the buttermilk and increase the speed to medium. Mix on medium high for 2 minutes to aerate the batter. Scrape the bowl and beater.\nAdd the egg mixture in 3 batches, scraping the bowl between each addition. Pour the batter into the prepared pan.\nBake until the cake springs back when lightly pressed or a toothpick inserted into the center comes out clean, about 40 minutes.\nCool for 10 minutes in the pan. Invert the cake onto a cooling rack set over a clean sheet pan. Cool until slightly warm before glazing.\nMake the Glaze\nCombine the sugar, vanilla and buttermilk in a small bowl and whisk until smooth.\nPour the glaze over the still slightly warm cake. You can scoop up the glaze from the sheet pan and use it to fill in any gaps in the glaze or leave it with the drips.\nCool completely and allow the glaze to set. Transfer to a serving plate.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.blueapron import BlueApron
from tests import ScraperTest


class TestBlueApronScraper(ScraperTest):

    scraper_class = BlueApron

    def test_host(self):
        self.assertEqual("blueapron.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.blueapron.com/recipes/bbq-chickpeas-farro-with-corn-cucumbers-hard-boiled-eggs-3",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            "BBQ Chickpeas & Farro with Corn, Cucumbers & Hard-Boiled Eggs",
            self.harvester_class.title(),
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Blue Apron")

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://media.blueapron.com/recipes/24483/c_main_dish_images/1593115477-32-0038-5165/0819_2PV2_BBQ-Chickpea_4449_CropRight_Web_high_feature.jpg",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 Pasture-Raised Eggs",
                "1 15.5 -Ounce Can Chickpeas",
                "½ cup Semi-Pearled Farro",
                "2 ears Of Corn",
                "2 Persian Cucumbers",
                "1 bunch Chives",
                "¼ cup Barbecue Sauce",
                "1 Tbsp Apple Cider Vinegar",
                "1 oz Sliced Pickled Jalapeño Pepper",
                "1 Tbsp Barbecue Spice Blend (Smoked Paprika, Sweet Paprika, Ground Fennel Seeds, Ground Coriander, Garlic Powder & Light Brown Sugar)",
                "2 tsps Honey",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "1 Cook the farro Remove the honey from the refrigerator to bring to room temperature. Fill a medium pot 3/4 of the way up with salted water; cover and heat to boiling on high. Fill a separate, small pot 3/4 of the way up with salted water; cover and heat to boiling on high. Once the medium pot is boiling, add the farro. Cook, uncovered, 18 to 20 minutes, or until tender. Turn off the heat. Drain thoroughly and return to the pot.\n2 Prepare the ingredients & make the vinaigrette Meanwhile, wash and dry the fresh produce. Remove the husks and silks from the corn; cut the kernels off the cobs. Drain and rinse the chickpeas. Halve the cucumbers lengthwise; thinly slice crosswise. Roughly chop the pepper; thoroughly wash your hands immediately after handing. In a bowl, combine the sliced cucumbers, a drizzle of olive oil, and as much of the chopped pepper as you’d like, depending on how spicy you’d like the dish to be; season with salt and pepper. Set aside to marinate, stirring occasionally, at least 10 minutes. Taste, then season with salt and pepper if desired.Thinly slice the chives. In a separate bowl, whisk together the honey (kneading the packet before opening), vinegar, and 1 tablespoon of olive oil; season with salt and pepper.\n3 Cook & slice the eggs While the cucumbers marinate, carefully add the eggs to the small pot of boiling water. Cook 9 minutes for hard-boiled. Drain and rinse under cold water 30 seconds to 1 minute to stop the cooking process. When cool enough to handle, peel the cooked eggs; thinly slice. Season with salt and pepper.\n4 Char the corn In a large pan (nonstick, if you have one), heat 2 teaspoons of olive oil on medium-high until hot. Add the corn kernels in an even layer; season with salt and pepper. Cook, without stirring, 3 to 4 minutes, or until charred (be careful, as the corn may pop as it cooks). Continue to cook, stirring occasionally, 1 to 2 minutes, or until softened. Transfer to a bowl. Wipe out the pan.\n5 Cook the chickpeas & serve your dish In the same pan, heat 2 teaspoons of olive oil on medium-high until hot. Add the drained chickpeas and half the spice blend (you will have extra). Cook, stirring occasionally, 2 to 3 minutes, or until slightly softened. Add the barbecue sauce (carefully, as the liquid may splatter) and 1/4 cup of water. Cook, stirring occasionally, 2 to 3 minutes, or until the sauce is thickened. Turn off the heat. Taste, then season with salt and pepper if desired. To the pot of cooked farro, add the vinaigrette; stir to combine. Taste, then season with salt and pepper if desired. Serve the finished farro topped with the sliced eggs, marinated cucumbers, cooked chickpeas, and charred corn. Garnish with the sliced chives. Enjoy!\n1 Cook the farro Remove the honey from the refrigerator to bring to room temperature. Fill a medium pot 3/4 of the way up with salted water; cover and heat to boiling on high. Fill a separate, small pot 3/4 of the way up with salted water; cover and heat to boiling on high. Once the medium pot is boiling, add the farro. Cook, uncovered, 18 to 20 minutes, or until tender. Turn off the heat. Drain thoroughly and return to the pot.\n2 Prepare the ingredients & make the vinaigrette Meanwhile, wash and dry the fresh produce. Remove the husks and silks from the corn; cut the kernels off the cobs. Drain and rinse the chickpeas. Halve the cucumbers lengthwise; thinly slice crosswise. Roughly chop the pepper; thoroughly wash your hands immediately after handing. In a bowl, combine the sliced cucumbers, a drizzle of olive oil, and as much of the chopped pepper as you’d like, depending on how spicy you’d like the dish to be; season with salt and pepper. Set aside to marinate, stirring occasionally, at least 10 minutes. Taste, then season with salt and pepper if desired.Thinly slice the chives. In a separate bowl, whisk together the honey (kneading the packet before opening), vinegar, and 1 tablespoon of olive oil; season with salt and pepper.\n3 Cook & slice the eggs While the cucumbers marinate, carefully add the eggs to the small pot of boiling water. Cook 9 minutes for hard-boiled. Drain and rinse under cold water 30 seconds to 1 minute to stop the cooking process. When cool enough to handle, peel the cooked eggs; thinly slice. Season with salt and pepper.\n4 Char the corn In a large pan (nonstick, if you have one), heat 2 teaspoons of olive oil on medium-high until hot. Add the corn kernels in an even layer; season with salt and pepper. Cook, without stirring, 3 to 4 minutes, or until charred (be careful, as the corn may pop as it cooks). Continue to cook, stirring occasionally, 1 to 2 minutes, or until softened. Transfer to a bowl. Wipe out the pan.\n5 Cook the chickpeas & serve your dish In the same pan, heat 2 teaspoons of olive oil on medium-high until hot. Add the drained chickpeas and half the spice blend (you will have extra). Cook, stirring occasionally, 2 to 3 minutes, or until slightly softened. Add the barbecue sauce (carefully, as the liquid may splatter) and 1/4 cup of water. Cook, stirring occasionally, 2 to 3 minutes, or until the sauce is thickened. Turn off the heat. Taste, then season with salt and pepper if desired. To the pot of cooked farro, add the vinaigrette; stir to combine. Taste, then season with salt and pepper if desired. Serve the finished farro topped with the sliced eggs, marinated cucumbers, cooked chickpeas, and charred corn. Garnish with the sliced chives. Enjoy!",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.misya import Misya
from tests import ScraperTest


class TestMisya(ScraperTest):

    scraper_class = Misya

    def test_host(self):
        self.assertEqual("misya.info", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.misya.info/ricetta/tortino-cuore-caldo.htm",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Tortino cuore caldo")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Flavia Imperatore")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.misya.info/wp-content/uploads/2016/02/Tortino-cuore-caldo1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 di uova",
                "2 di tuorli",
                "100 gr di zucchero",
                "110 gr di cioccolato bianco",
                "110 gr di burro",
                "50 gr di farina",
                "6 gr di cacao",
                "1 cucchiaino di essenza di vaniglia",
                "1 cucchiaino di colorante rosso",
                "zucchero a velo",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Montate i tuorli e le uova fino a renderle spumose.\nPoi aggiungete lo zucchero\nPoi unite il cioccolato ed il burro fuso.\nAggiungete adesso il cacao e la farina.\nQuindi aggiungete il colorante e la vaniglia e mescolate fino ad ottenere un impasto di un bel colore rosso.\nVersate l'impasto in 6 stampini imburrati ed infarinati meticolosamente.\nPoi infornate i tortini in forno già caldo a 190° e cuocete per 13-15 minuti circa.Vi accorgerete quando è il momento giusto quando vedrete una leggera crosticina sui bordi e la superficie ma muovendo lo stampino risulterà ancora morbido.Fate la prova con uno prima di tirarli via dal forno tutti e ricordatevi che ogni forno è diverso dall'altro.\nLasciate riposare un minuto, poi capovolgete su un piatto da dessert.\nServite il Tortino cuore caldo immediatamente spolverizzandolo con zucchero a velo.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.8, self.harvester_class.ratings())



from recipe_scrapers.halfbakedharvest import HalfBakedHarvest
from tests import ScraperTest


class TestHalfBakedHarvestScraper(ScraperTest):

    scraper_class = HalfBakedHarvest

    def test_host(self):
        self.assertEqual("halfbakedharvest.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.halfbakedharvest.com/brown-butter-corn-and-feta-orzo/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Brown Butter Corn and Feta Orzo with Crispy Prosciutto",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "halfbakedharvest")

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.halfbakedharvest.com/wp-content/uploads/2020/07/Brown-Butter-Corn-and-Feta-Orzo-with-Crispy-Prosciutto-1-1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 pound dry orzo pasta",
                "3 ounces prosciutto, torn",
                "4 tablespoon salted butter",
                "2 tablespoons extra virgin olive oil",
                "1 shallot, finely chopped",
                "1 jalapeño, seeded if desired, and chopped",
                "2 tablespoons fresh thyme leaves",
                "2 tablespoons fresh lemon juice",
                "2 tablespoons champagne or apple cider vinegar",
                "2 teaspoons honey",
                "1/2 cup fresh basil, roughly chopped",
                "1/4 cup fresh chopped chives",
                "4 ears grilled or steamed corn, kernels removed from the cobb",
                "2 cups cherry tomatoes, halved",
                "8 ounces crumbled feta cheese",
                "2 avocados, sliced",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """1. Bring a large pot of salted water to a boil. Boil the pasta to al dente, according to package directions. Drain and add the pasta right back to the pot. 2. Meanwhile, cook the prosciutto in a large skillet set over medium heat until crispy, about 2 minutes per side. Remove the prosciutto from the skillet. 3. To the skillet, add the butter. Allow the butter to brown until it smells toasted and is a deep golden color, about 3-4 minutes. Stir in the olive oil, shallot, jalapeño, and thyme. Cook 1-2 minutes, then remove from the heat. Pour the browned butter over the hot orzo. Toss to combine. 4. To the orzo, add the basil, chives, lemon juice, vinegar, and honey. Season with salt and pepper and toss. Stir in the corn, tomatoes, and feta. Top the pasta with crispy prosciutto and avocado. Serve warm or at room temp.""",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.farmhousedelivery import FarmhouseDelivery
from tests import ScraperTest


class TestFarmhouseDeliveryScraper(ScraperTest):

    scraper_class = FarmhouseDelivery
    test_file_name = "farmhousedelivery_1"

    def test_host(self):
        self.assertEqual("recipes.farmhousedelivery.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "http://recipes.farmhousedelivery.com/green-shakshuka/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Green Shakshuka", self.harvester_class.title())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 yellow onion, slivered",
                "3 cloves garlic, minced",
                "2 Tbsp. olive oil",
                "1 jalapeno, seeded and minced",
                "4 big handfuls greens (mix kale & spinach), chopped and washed",
                "1/2 c cream",
                "4 eggs",
                "Salt & pepper",
                "Plain yogurt, for serving",
                "Sriracha or other hot sauce, for serving",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Saute onions & garlic in olive oil in a large skillet until they take on a little color. Add jalapeno and continue cooking for 1-2 minutes. Add chopped greens, season with salt and pepper to taste and cover until greens are just wilted. Add cream and bring to a simmer. Crack eggs on top of greens, cover and cook until eggs are cooked to your preference. Serve in wide bowls with a dollop of yogurt and a drizzle of hot sauce and thick slices of warm bread on the side."
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_image(self):
        self.assertEqual(
            "http://recipesfhd.wpengine.com/wp-content/uploads/2020/04/no-blob_clean-handle-1024x683.jpg",
            self.harvester_class.image(),
        )



from recipe_scrapers.tudogostoso import TudoGostoso
from tests import ScraperTest


class TestTudoGostosoScraper(ScraperTest):

    scraper_class = TudoGostoso

    def test_host(self):
        self.assertEqual("tudogostoso.com.br", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.tudogostoso.com.br/receita/128825-caipirinha-original.html",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Caipirinha - Original")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Daniela Belizario")

    def test_total_time(self):
        self.assertEqual(5, self.harvester_class.total_time())

    def test_ingredients(self):
        self.assertListEqual(
            ["1 limão grande", "2 colheres de açúcar", "gelo a gosto", "cachaça"],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Pegue o limão coloque-o na horizontal e retire as duas pontas, vire-o na vertical e corte-o ao meio, retire os meio (parte branca) do limão e fatie.Coloque o limão fatiado e duas colheres bem cheias de açúcar dentro de um copo próprio para a bebida e com um socador esprema até que saia todo o suco do limão.Coloque pedras de gelo até quase encher o copo (aproximadamente 12 pedras pequenas de gelo) e encha o copo com a cachaça.Mexa bem com uma colher ou coloque em uma coqueteleira e sirva-se! Informações Adicionais Confira no TudoGostoso mais drinks deliciosos como batida de maracujá, mojito, licor caseiro, batida de morango e outras bebidas! Você sabia que existe mais de um tipo de caipirinha? É verdade! E aqui no TudoGostoso a gente te mostra e te ensina como preparar algumas delas. Confira e não deixe de fazer. Sabe qual a origem da caipirinha? Não? O TudoGostoso te mostra! Confira como essa e outros tipos de bebidas foram criados em uma matéria maravilhosa do blog. Veja também: Caipirinha perfeita: 4 dicas para arrasar nas festas Como fazer caipirinha de algodão-doce: confira esse e mais 2 drinques incríveis",
            self.harvester_class.instructions(),
        )
